# Copyright 2020 Cloudera Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import logging

import six

from thrift.protocol.TBinaryProtocol import TBinaryProtocolAccelerated
# noinspection PyProtectedMember
from impala._thrift_gen.ImpalaService import ImpalaHiveServer2Service

ThriftClient = ImpalaHiveServer2Service.Client

from impala import hiveserver2 as hs2
from impala._thrift_api import ImpalaHttpClient
from impala.error import HttpError
from impala.hiveserver2 import HS2Service
from impala.tests.util import ImpylaTestEnv

ENV = ImpylaTestEnv()


# This code is based on https://github.com/apache/impala/blob/master/tests/custom_cluster/test_hs2_fault_injection.py


class FaultInjectingHttpClient(ImpalaHttpClient, object):
    """Class for injecting faults into ImpalaHttpClient. Faults are injected by using the
    'enable_fault' method. The 'flush' method is overridden to check for injected faults
    and raise exceptions, if needed."""

    def __init__(self, *args, **kwargs):
        super(FaultInjectingHttpClient, self).__init__(*args, **kwargs)
        self.fault_code = None
        self.fault_message = None
        self.fault_enabled = False
        self.num_requests = 0
        self.fault_frequency = 0

    # noinspection PyAttributeOutsideInit
    def enable_fault(self, http_code, http_message, fault_frequency, fault_body=None,
                     fault_headers=None, set_num_requests=None):
        """Inject fault with given code and message at the given frequency.
        As an example, if frequency is 20% then inject fault for 1 out of every 5
        requests."""
        if fault_headers is None:
            fault_headers = {}
        self.fault_enabled = True
        self.fault_code = http_code
        self.fault_message = http_message
        self.fault_frequency = fault_frequency
        assert 0 < fault_frequency <= 1
        if set_num_requests is not None:
            self.num_requests = set_num_requests
        else:
            self.num_requests = 0
        self.fault_body = fault_body
        self.fault_headers = fault_headers

    def disable_fault(self):
        self.fault_enabled = False

    def _check_code(self):
        if self.code >= 300:
            # Report any http response code that is not 1XX (informational response) or
            # 2XX (successful).
            raise HttpError(self.code, self.message, self.body, self.headers)

    def _inject_fault(self):
        if not self.fault_enabled:
            return False
        if self.fault_frequency == 1:
            return True
        if round(self.num_requests % (1 / self.fault_frequency)) == 1:
            return True
        return False

    # noinspection PyAttributeOutsideInit
    def flush(self):
        ImpalaHttpClient.flush(self)
        self.num_requests += 1
        # Override code and message with the injected fault
        if self.fault_code is not None and self._inject_fault():
            self.code = self.fault_code
            self.message = self.fault_message
            self.body = self.fault_body
            self.headers = self.fault_headers
            self._check_code()

    def _read(self, sz):
        """Keep pep8 quiet"""
        pass


# noinspection PyMethodMayBeStatic
class TestHS2FaultInjection(object):
    """Class for testing the http fault injection in various rpcs used by Impyla"""

    def setup_method(self):
        host = "[%s]" % ENV.host if ":" in ENV.host else ENV.host
        url = 'http://%s:%s/%s' % (host, ENV.http_port, "cliservice")
        self.transport = FaultInjectingHttpClient(url)
        self.configuration = {'idle_session_timeout': '30'}

    def teardown_method(self):
        self.transport.disable_fault()

    def connect(self, retries=3):
        self.transport.open()
        protocol = TBinaryProtocolAccelerated(self.transport)
        service = ThriftClient(protocol)
        service = HS2Service(service, retries=retries)
        return hs2.HiveServer2Connection(service, default_db=None)

    def __expect_msg_retry(self, impala_rpc_name, retries):
        """Returns expected log message for rpcs which can be retried"""
        return ("Caught HttpError HTTP code 502: Injected Fault  in {0} (tries_left={1})".
                format(impala_rpc_name, retries))

    def __expect_msg_retry_with_extra(self, impala_rpc_name, retries):
        """Returns expected log message for rpcs which can be retried and where the http
        message has a message body"""
        return ("Caught HttpError HTTP code 503: Injected Fault EXTRA in {0} (tries_left={1})".
                format(impala_rpc_name, retries))

    def __expect_msg_retry_with_retry_after(self, impala_rpc_name, retries):
        """Returns expected log message for rpcs which can be retried and where the http
        message has a body and a Retry-After header that can be correctly decoded"""
        return ("Caught HttpError HTTP code 503: Injected Fault EXTRA in {0} (tries_left={1}), retry after 1 secs".
                format(impala_rpc_name, retries))

    def __expect_msg_retry_with_retry_after_sleep(self):
        """Returns expected log message for the sleep which uses a value
        from the Retry-After header"""
        return ("sleeping after seeing Retry-After value of 1")

    def __expect_msg_retry_after_default_sleep(self):
        """Returns expected log message for the default sleep time of 1 second"""
        return ("sleeping for 1 second before retrying")

    def __expect_msg_retry_with_retry_after_no_extra(self, impala_rpc_name, retries):
        """Returns expected log message for rpcs which can be retried and the http
        message has a Retry-After header that can be correctly decoded"""
        return ("Caught HttpError HTTP code 503: Injected Fault  in {0} (tries_left={1}), retry after 1 secs".
                format(impala_rpc_name, retries))
  
    def __expect_msg_no_retry(self, impala_rpc_name):
        """Returns expected log message for rpcs which can not be retried"""
        return ("Caught HttpError HTTP code 502: Injected Fault  in {0} which is not retryable".
                format(impala_rpc_name))

    def test_connect(self, caplog):
        """Tests fault injection in cursor() call.
        OpenSession rpc fails.
        Retries results in a successful connection."""
        caplog.set_level(logging.DEBUG)
        self.transport.enable_fault(502, "Injected Fault", 0.2)
        con = self.connect(retries=4)
        cur = con.cursor(configuration=self.configuration)
        cur.close()
        con.close()
        assert self.__expect_msg_retry("OpenSession", 4) in caplog.text

    def test_connect_proxy(self, caplog):
        """Tests fault injection in cursor() call.
        The injected error has a message body.
        OpenSession rpc fails.
        Retries results in a successful connection."""
        caplog.set_level(logging.DEBUG)
        self.transport.enable_fault(503, "Injected Fault", 0.20, 'EXTRA')
        con = self.connect(retries=4)
        cur = con.cursor(configuration=self.configuration)
        cur.close()
        con.close()
        assert self.__expect_msg_retry_with_extra("OpenSession", 4) in caplog.text
        assert self.__expect_msg_retry_after_default_sleep() in caplog.text

    def test_connect_proxy_no_retry(self, caplog):
        """Tests fault injection in cursor() call.
        The injected error contains headers but no Retry-After header.
        OpenSession rpc fails.
        Retries results in a successful connection."""
        caplog.set_level(logging.DEBUG)
        self.transport.enable_fault(503, "Injected Fault", 0.20, 'EXTRA',
                                    {"header1": "value1"})
        con = self.connect(retries=4)
        cur = con.cursor(configuration=self.configuration)
        cur.close()
        con.close()
        assert self.__expect_msg_retry_with_extra("OpenSession", 4) in caplog.text
        assert self.__expect_msg_retry_after_default_sleep() in caplog.text

    def test_connect_proxy_bad_retry(self, caplog):
        """Tests fault injection in cursor() call.
        The injected error contains a body and a junk Retry-After header.
        OpenSession rpc fails.
        Retries results in a successful connection."""
        caplog.set_level(logging.DEBUG)
        self.transport.enable_fault(503, "Injected Fault", 0.20, 'EXTRA',
                                    {"header1": "value1",
                                     "Retry-After": "junk"})
        con = self.connect(retries=4)
        cur = con.cursor(configuration=self.configuration)
        cur.close()
        con.close()
        assert self.__expect_msg_retry_with_extra("OpenSession", 4) in caplog.text
        assert self.__expect_msg_retry_after_default_sleep() in caplog.text

    def test_connect_proxy_retry(self, caplog):
        """Tests fault injection in cursor() call.
        The injected error contains a body and a Retry-After header that can be decoded.
        Retries results in a successful connection."""
        caplog.set_level(logging.DEBUG)
        self.transport.enable_fault(503, "Injected Fault", 0.20, 'EXTRA',
                                    {"header1": "value1",
                                     "Retry-After": "1"})
        con = self.connect(retries=4)
        cur = con.cursor(configuration=self.configuration)
        cur.close()
        con.close()
        assert self.__expect_msg_retry_with_retry_after("OpenSession", 4) in caplog.text
        assert self.__expect_msg_retry_with_retry_after_sleep() in caplog.text

    def test_connect_proxy_retry_no_body(self, caplog):
        """Tests fault injection in cursor() call.
        The injected error has no body but does have a Retry-After header that can be decoded.
        Retries results in a successful connection."""
        caplog.set_level(logging.DEBUG)
        self.transport.enable_fault(503, "Injected Fault", 0.20, None,
                                    {"header1": "value1",
                                     "Retry-After": "1"})
        con = self.connect(retries=4)
        cur = con.cursor(configuration=self.configuration)
        cur.close()
        con.close()
        assert self.__expect_msg_retry_with_retry_after_no_extra("OpenSession", 4) in caplog.text

    def test_execute_query(self, caplog):
        """Tests fault injection in execute().
        ExecuteStatement rpc fails and results in error since retries are not supported."""
        con = self.connect()
        cur = con.cursor(configuration=self.configuration)
        caplog.set_level(logging.DEBUG)
        self.transport.enable_fault(502, "Injected Fault", 0.50)

        query_handle = None
        try:
            query_handle = cur.execute('select 1')
            assert False, 'execute should have failed'
        except HttpError as e:
            assert str(e) == 'HTTP code 502: Injected Fault'
        assert query_handle is None
        cur.close()
        con.close()
        assert self.__expect_msg_no_retry("ExecuteStatement") in caplog.text

    def test_get_query_state(self, caplog):
        """Tests fault injection in fetchall().
        GetOperationStatus rpc fails but is retried successfully."""
        con = self.connect(retries=4)
        cur = con.cursor(configuration=self.configuration)
        caplog.set_level(logging.DEBUG)
        cur.execute_async('select 1', {})
        self.transport.enable_fault(502, "Injected Fault", 0.1)
        cur.fetchall()
        cur.close()
        con.close()
        assert self.__expect_msg_retry("GetOperationStatus", 4) in caplog.text

    def test_get_result_set_metadata(self, caplog):
        """Tests fault injection in fetchcbatch().
        GetResultSetMetadata rpc fails and is retried succesfully."""
        con = self.connect(retries=4)
        cur = con.cursor(configuration=self.configuration)
        caplog.set_level(logging.DEBUG)
        cur.execute('select 1', {})
        self.transport.enable_fault(502, "Injected Fault", 0.1)
        cur.fetchcbatch()
        cur.close()
        con.close()
        assert self.__expect_msg_retry("GetResultSetMetadata", 4) in caplog.text

    def test_fetch_results(self, caplog):
        """Tests fault injection in fetchcbatch().
        FetchResults rpc fails and cannot be retried."""
        con = self.connect()
        cur = con.cursor(configuration=self.configuration)
        caplog.set_level(logging.DEBUG)
        cur.execute('select 1', {})
        try:
            self.transport.enable_fault(502, "Injected Fault", 0.5, set_num_requests=1)
            cur.fetchcbatch()
            assert False, 'should see exception'
        except HttpError as e:
            assert str(e) == 'HTTP code 502: Injected Fault'
        self.transport.disable_fault()
        cur.close()
        con.close()
        assert self.__expect_msg_no_retry("FetchResults") in caplog.text

    def test_close_operation(self, caplog):
        """Tests fault injection in fetchcbatch().
        CloseOperation rpc fails and cannot be retried.."""
        con = self.connect()
        cur = con.cursor(configuration=self.configuration)
        caplog.set_level(logging.DEBUG)
        cur.execute('select 1', {})
        cur.fetchcbatch()
        try:
            self.transport.enable_fault(502, "Injected Fault", 0.5)
            cur.close()
        except HttpError as e:
            assert str(e) == 'HTTP code 502: Injected Fault'
        self.transport.disable_fault()
        cur.close()
        con.close()
        assert self.__expect_msg_no_retry("CloseImpalaOperation") in caplog.text

    def test_get_runtime_profile_summary(self, caplog):
        """Tests fault injection in get_profile(), get_summary(), and get_log().
        GetRuntimeProfile, GetExecSummary and GetLog rpcs fail due to fault, but succeed
        after retries"""
        con = self.connect(retries=4)
        cur = con.cursor(configuration=self.configuration)
        caplog.set_level(logging.DEBUG)
        cur.execute('select 1', {})
        cur.fetchcbatch()
        self.transport.enable_fault(502, "Injected Fault", 0.50)
        profile = cur.get_profile()
        assert profile is not None
        summary = cur.get_summary()
        assert summary is not None
        ret_log = cur.get_log()
        assert ret_log is not None
        self.transport.disable_fault()
        cur.close()
        con.close()
        assert self.__expect_msg_retry("GetRuntimeProfile", 4) in caplog.text
        assert self.__expect_msg_retry("GetExecSummary", 4) in caplog.text
        assert self.__expect_msg_retry("GetLog", 4) in caplog.text



# Copyright 2019 Cloudera Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import sys

import pytest
from impala.compat import _xrange as xrange
from pytest import fixture

BIGGER_TABLE_NUM_ROWS = 100

@fixture(scope='module')
def bigger_table(cur):
    table_name = 'tmp_bigger_table'
    ddl = """CREATE TABLE {0} (s string)
             STORED AS PARQUET""".format(table_name)
    cur.execute(ddl)
    dml = """INSERT INTO {0}
             VALUES {1}""".format(table_name,
                 ",".join(["('row{0}')".format(i) for i in xrange(BIGGER_TABLE_NUM_ROWS)]))
    # Disable codegen and expr rewrites so query runs faster.
    cur.execute("set disable_codegen=1")
    cur.execute("set enable_expr_rewrites=0")
    cur.execute(dml)
    try:
        yield table_name
    finally:
        cur.execute("DROP TABLE {0}".format(table_name))


def test_has_more_rows(cur, bigger_table):
    """Test that impyla correctly handles empty row batches returned with the
    hasMoreRows flag."""
    # Set the fetch timeout very low and add sleeps so that Impala will return
    # empty batches. Run on a single node with a single thread to make as predictable
    # as possible.
    cur.execute("set fetch_rows_timeout_ms=1")
    cur.execute("set num_nodes=1")
    cur.execute("set mt_dop=1")
    cur.execute("""select *
                   from {0}
                   where s != cast(sleep(2) as string)""".format(bigger_table))
    expected_rows = [("row{0}".format(i),) for i in xrange(BIGGER_TABLE_NUM_ROWS)]
    assert sorted(cur.fetchall()) == sorted(expected_rows)

@fixture(scope='function')
def empty_table(cur):
    table_name = 'tmp_empty_table'
    ddl = """CREATE TABLE {0} (i int)""".format(table_name)
    cur.execute(ddl)
    try:
        yield table_name
    finally:
        cur.execute("DROP TABLE {0}".format(table_name))

def test_dml_rowcount(cur, empty_table):
    """Test that impyla correctly sets rowcount for insert statements."""
    dml = """INSERT INTO {0}
	     VALUES (0)""".format(empty_table)
    cur.execute(dml)
    assert cur.rowcount == 1

def test_row_count_in_empty_result(cur, empty_table):
    """Test that impyla correctly sets rowcount when 0 rows are returned.
       This case is missing from dbapi2 compliance tests.
    """
    query = """SELECT * FROM {0}""".format(empty_table)
    cur.execute(query)
    cur.fetchall()
    assert cur.rowcount == 0

def test_get_log(cur, empty_table):
    """Test that impyla can return the result of get_log after the query
       is closed.
    """
    query = """SELECT * FROM {0}""".format(empty_table)
    for mt_dop in ['0', '2']:
        cur.execute(query, configuration={'mt_dop': mt_dop})
        cur.fetchall()
        validate_log(cur)
        cur.close_operation()

def validate_log(cur):
    # The query should be closed at this point.
    assert not cur._last_operation_active
    log = cur.get_log()
    assert "100% Complete" in log
    # Also check that summary and runtime profile are available
    summary = cur.get_summary()
    assert summary is not None
    for node in summary.nodes:
        assert hasattr(node, 'node_id')
        assert hasattr(node, 'fragment_idx')
        assert hasattr(node, 'label')
        assert hasattr(node, 'label_detail')
        assert hasattr(node, 'num_children')
        assert hasattr(node, 'estimated_stats')
        assert hasattr(node, 'exec_stats')
        assert hasattr(node, 'is_broadcast')
        assert hasattr(node, 'num_hosts')
        assert node.num_hosts > 0
        assert len(node.exec_stats) >= node.num_hosts
    profile = cur.get_profile()
    assert profile is not None



# -*- coding: UTF-8 -*-
# Copyright 2015 Cloudera Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import datetime
import pytest
import sys
from pytest import fixture
from decimal import Decimal

@fixture(scope='module')
def decimal_table(cur):
    table_name = 'tmp_decimal_table'
    ddl = """CREATE TABLE {0} (
                 f1 decimal(10, 2),
                 f2 decimal(7, 5),
                 f3 decimal(38, 17))""".format(table_name)
    cur.execute(ddl)
    try:
        yield table_name
    finally:
        cur.execute("DROP TABLE {0}".format(table_name))


@pytest.mark.connect
def test_cursor_description_precision_scale(cur, decimal_table):
    # According to the DBAPI 2.0, these are the 7 fields of cursor.description
    # - name
    # - type_code
    # - display_size
    # - internal_size
    # - precision
    # - scale
    # - null_ok
    expected = [(10, 2),
                (7, 5),
                (38, 17)]
    cur.execute('select * from {0} limit 0'.format(decimal_table))
    observed = [(t[4], t[5]) for t in cur.description]
    for (exp, obs) in zip(expected, observed):
        assert exp == obs


@fixture(scope='module')
def decimal_table2(cur):
    table_name = 'tmp_decimal_table2'
    ddl = """CREATE TABLE {0} (val decimal(18, 9))""".format(table_name)
    cur.execute(ddl)
    cur.execute('''insert into {0}
                   values (cast(123456789.123456789 as decimal(18, 9))),
                          (cast(-123456789.123456789 as decimal(18, 9))),
                          (cast(0.000000001 as decimal(18, 9))),
                          (cast(-0.000000001 as decimal(18, 9))),
                          (cast(999999999.999999999 as decimal(18, 9))),
                          (cast(-999999999.999999999 as decimal(18, 9))),
                          (NULL)'''.format(table_name))
    try:
        yield table_name
    finally:
        cur.execute("DROP TABLE {0}".format(table_name))


def common_test_decimal(cur, decimal_table):
    """Read back a few decimal values in a wide range."""
    cur.execute('select val from {0} order by val'.format(decimal_table))
    results = cur.fetchall()
    assert results == [(Decimal('-999999999.999999999'),),
                       (Decimal('-123456789.123456789'),),
                       (Decimal('-0.000000001'),),
                       (Decimal('0.000000001'),),
                       (Decimal('123456789.123456789'),),
                       (Decimal('999999999.999999999'),),
                       (None,)]


@pytest.mark.connect
def test_decimal_basic(cur, decimal_table2):
    common_test_decimal(cur, decimal_table2)


@pytest.mark.connect
def test_decimal_no_string_conv(cur_no_string_conv, decimal_table2):
    common_test_decimal(cur_no_string_conv, decimal_table2)


@fixture(scope='module')
def date_table(cur):
    table_name = 'tmp_date_table'
    ddl = """CREATE TABLE {0} (d date)""".format(table_name)
    cur.execute(ddl)
    cur.execute('''insert into {0}
                   values (date "0001-01-01"), (date "1999-9-9")'''.format(table_name))
    try:
        yield table_name
    finally:
        cur.execute("DROP TABLE {0}".format(table_name))


def common_test_date(cur, date_table):
    """Read back a couple of data values in a wide range."""
    cur.execute('select d from {0} order by d'.format(date_table))
    results = cur.fetchall()
    assert results == [(datetime.date(1, 1, 1),), (datetime.date(1999, 9, 9),)]


@pytest.mark.connect
def test_date_basic(cur, date_table):
    common_test_date(cur, date_table)


@pytest.mark.connect
def test_date_no_string_conv(cur_no_string_conv, date_table):
    common_test_date(cur_no_string_conv, date_table)


@fixture(scope='module')
def timestamp_table(cur):
    table_name = 'tmp_timestamp_table'
    ddl = """CREATE TABLE {0} (ts timestamp)""".format(table_name)
    cur.execute(ddl)
    cur.execute('''insert into {0}
                   values (cast("1400-01-01 00:00:00" as timestamp)),
                          (cast("2014-06-23 13:30:51" as timestamp)),
                          (cast("2014-06-23 13:30:51.123" as timestamp)),
                          (cast("2014-06-23 13:30:51.123456" as timestamp)),
                          (cast("2014-06-23 13:30:51.123456789" as timestamp)),
                          (cast("9999-12-31 23:59:59" as timestamp))'''.format(table_name))
    try:
        yield table_name
    finally:
        cur.execute("DROP TABLE {0}".format(table_name))


def common_test_timestamp(cur, timestamp_table):
    """Read back a few timestamp values in a wide range."""
    cur.execute('select ts from {0} order by ts'.format(timestamp_table))
    results = cur.fetchall()
    assert results == [(datetime.datetime(1400, 1, 1, 0, 0),),
                       (datetime.datetime(2014, 6, 23, 13, 30, 51),),
                       (datetime.datetime(2014, 6, 23, 13, 30, 51, 123000),),
                       (datetime.datetime(2014, 6, 23, 13, 30, 51, 123456),),
                       (datetime.datetime(2014, 6, 23, 13, 30, 51, 123456),),
                       (datetime.datetime(9999, 12, 31, 23, 59, 59),)]


@pytest.mark.connect
def test_timestamp_basic(cur, timestamp_table):
    common_test_timestamp(cur, timestamp_table)


@pytest.mark.connect
def test_timestamp_no_string_conv(cur_no_string_conv, timestamp_table):
    common_test_timestamp(cur_no_string_conv, timestamp_table)


@pytest.mark.connect
def test_utf8_strings(cur):
    """Use STRING/VARCHAR/CHAR values  with multi byte unicode code points in a query."""
    cur.execute('select "引擎", cast("引擎" as varchar(6)), cast("引擎" as char(6))')
    result = cur.fetchone()
    assert result == ("引擎",) * 3

    # Tests returning STRING/VARCHAR/CHAR strings that are not valid UTF-8.
    # With Python 3 and Thrift 0.11.0 these tests needed TCLIService.thrift to be
    # modified. Syncing thrift files from Hive/Impala is likely to break these tests.
    cur.execute('select substr("引擎", 1, 4), cast("引擎" as varchar(4)), cast("引擎" as char(4))')
    result = cur.fetchone()
    assert result == (b"\xe5\xbc\x95\xe6",) * 3
    assert result[0].decode("UTF-8", "replace") == u"引�"

    cur.execute('select unhex("AA")')
    result = cur.fetchone()[0]
    assert result == b"\xaa"
    assert result.decode("UTF-8", "replace") == u"�"


@pytest.mark.connect
def test_string_conv(cur):
    cur.execute('select "Test string"')
    result = cur.fetchone()
    is_unicode = isinstance(result[0], str)


@pytest.mark.connect
def test_string_no_string_conv(cur_no_string_conv):
    cur = cur_no_string_conv
    cur.execute('select "Test string"')
    result = cur.fetchone()

    if sys.version_info[0] < 3:
        assert isinstance(result[0], str)
    else:
        assert isinstance(result[0], bytes)



# Copyright 2019 Cloudera Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import socket
import threading
from contextlib import closing

import pytest
import requests
import six
from six.moves import SimpleHTTPServer
from six.moves import http_client
from six.moves import socketserver

from impala.error import HttpError
from impala.tests.util import ImpylaTestEnv, is_ipv6_only_host

ENV = ImpylaTestEnv()

IS_IPV6_ONLY_HOST = is_ipv6_only_host(ENV.host, ENV.port)
LOCAL_HOST = "::1" if IS_IPV6_ONLY_HOST else "127.0.0.1"

# socketserver.TCPServer cannot listen both on ipv4 and ipv6. Listen to ipv6
# if the hs2-http server has only ipv6 address.
class IPv4or6TcpServer(socketserver.TCPServer):
  address_family = socket.AF_INET6 if IS_IPV6_ONLY_HOST else socket.AF_INET
  def __init__(self, host_port, req_handler):
    socketserver.TCPServer.__init__(self, host_port, req_handler)

@pytest.fixture
def http_503_server():
  class RequestHandler503(SimpleHTTPServer.SimpleHTTPRequestHandler):
    """A custom http handler that checks for duplicate 'Host' headers from the most
    recent http request, and always returns a 503 http code"""

    def do_POST(self):
      # Ensure that only one 'Host' header is contained in the request before responding.
      request_headers = None
      host_hdr_count = 0
      if six.PY2:
        # The unfortunately named self.headers here is an instance of mimetools.Message that
        # contains the request headers.
        request_headers = self.headers.headers
        host_hdr_count = sum([header.startswith('Host:') for header in request_headers])
      if six.PY3:
        # In Python3 self.Headers is an HTTPMessage.
        request_headers = self.headers
        host_hdr_count = sum([header[0] == 'Host' for header in request_headers.items()])
      assert host_hdr_count == 1, "need single 'Host:' header in %s" % request_headers

      # Respond with 503.
      self.send_response(code=http_client.SERVICE_UNAVAILABLE, message="Service Unavailable")
      self.end_headers()
      self.wfile.write("extra text".encode('utf-8'))

  class TestHTTPServer503(object):
    def __init__(self):
      self.HOST = LOCAL_HOST
      self.PORT = get_unused_port()
      self.httpd = IPv4or6TcpServer((self.HOST, self.PORT), RequestHandler503)

      self.http_server_thread = threading.Thread(target=self.httpd.serve_forever)
      self.http_server_thread.start()

  server = TestHTTPServer503()
  yield server

  # Cleanup after test.
  shutdown_server(server)


@pytest.yield_fixture
def http_proxy_server():
  """A fixture that creates a reverse http proxy."""

  class RequestHandlerProxy(SimpleHTTPServer.SimpleHTTPRequestHandler):
    """A custom http handler that acts as a reverse http proxy. This proxy will forward
    http messages to Impala, and copy the responses back to the client. In addition, it
    will save the outgoing http message headers in a class variable so that they can be
    accessed by test code."""

    # This class variable is used to store the most recently seen outgoing http
    # message headers.
    saved_headers=None

    def __init__(self, request, client_address, server):
      SimpleHTTPServer.SimpleHTTPRequestHandler.__init__(self, request, client_address,
                                                    server)

    def do_POST(self):
      # Read the body of the incoming http post message.
      data_string = self.rfile.read(int(self.headers['Content-Length']))
      # Save the http headers from the message in a class variable.
      RequestHandlerProxy.saved_headers = self.decode_raw_headers()
      # Forward the http post message to Impala and get a response message.
      host = "[%s]" % ENV.host  if ":" in ENV.host else ENV.host
      response = requests.post(
        url="http://{0}:{1}/cliservice".format(host, ENV.http_port),
        headers=self.headers, data=data_string)
      # Send the response message back to the client.
      self.send_response(code=response.status_code)
      # Send the http headers.
      # In python3 response.headers is a CaseInsensitiveDict
      # In python2 response.headers is a dict
      for key, value in response.headers.items():
        self.send_header(keyword=key, value=value)
      self.end_headers()
      # Send the message body.
      self.wfile.write(response.content)

    def decode_raw_headers(self):
      """Decode a list of header strings into a list of tuples, each tuple containing a
      key-value pair. The details of how to get the headers differs between Python2
      and Python3"""
      if six.PY2:
        header_list = []
        # In Python2 self.headers is an instance of mimetools.Message and
        # self.headers.headers is a list of raw header strings.
        # An example header string: 'Accept-Encoding: identity\\r\\n'
        for header in self.headers.headers:
          stripped = header.strip()
          key, value = stripped.split(':', 1)
          header_list.append((key.strip(), value.strip()))
        return header_list
      if six.PY3:
        # In Python 3 self.headers._headers is what we need
        return self.headers._headers


  class TestHTTPServerProxy(object):
    def __init__(self, clazz):
      self.clazz = clazz
      self.HOST = LOCAL_HOST
      self.PORT = get_unused_port()
      self.httpd = IPv4or6TcpServer((self.HOST, self.PORT), clazz)
      self.http_server_thread = threading.Thread(target=self.httpd.serve_forever)
      self.http_server_thread.start()

    def get_headers(self):
      """Return the most recently seen outgoing http message headers."""
      return self.clazz.saved_headers

  server = TestHTTPServerProxy(RequestHandlerProxy)
  yield server

  # Cleanup after test.
  shutdown_server(server)


from impala.dbapi import connect

class TestHttpConnect(object):
  def test_simple_connect(self):
    con = connect(ENV.host, ENV.http_port, use_http_transport=True, http_path="cliservice")
    cur = con.cursor()
    cur.execute('select 1')
    rows = cur.fetchall()
    assert rows == [(1,)]

  def test_http_interactions(self, http_503_server):
    """Test interactions with the http server when using hs2-http protocol.
    Check that there is an HttpError exception when the server returns a 503 error."""
    con = connect(ENV.host, http_503_server.PORT, use_http_transport=True)
    try:
      con.cursor()
      assert False, "Should have got exception"
    except HttpError as e:
      assert str(e) == "HTTP code 503: Service Unavailable"
      assert e.code == http_client.SERVICE_UNAVAILABLE
      assert e.body.decode("utf-8") == "extra text"

  def test_duplicate_headers(self, http_proxy_server):
    """Test that we can use 'connect' with the get_user_custom_headers_func parameter
    to add duplicate http message headers to outgoing messages."""
    con = connect(ENV.host, http_proxy_server.PORT, use_http_transport=True,
                  get_user_custom_headers_func=get_user_custom_headers_func)
    cur = con.cursor()
    cur.execute('select 1')
    rows = cur.fetchall()
    assert rows == [(1,)]

    # Get the outgoing message headers from the last outgoing http message.
    headers = http_proxy_server.get_headers()
    # For sanity test the count of a few simple expected headers.
    assert count_tuples_with_key(headers, "Host") == 1
    assert count_tuples_with_key(headers, "User-Agent") == 1
    # Check that the custom headers are present.
    assert count_tuples_with_key(headers, "key1") == 2
    assert count_tuples_with_key(headers, "key2") == 1
    assert count_tuples_with_key(headers, "key3") == 0
    
  def test_basic_auth_headers(self, http_proxy_server):
    con = connect(
      ENV.host,
      http_proxy_server.PORT,
      use_http_transport=True,
      user="thisisaratherlongusername",
      password="very!long!passwordthatcreatesalongbasic64encoding",
      auth_mechanism="PLAIN"
    )
    cur = con.cursor()
    cur.execute('select 1')
    rows = cur.fetchall()
    assert rows == [(1,)]

    headers = http_proxy_server.get_headers()
    assert ('Authorization', "Basic dGhpc2lzYXJhdGhlcmxvbmd1c2VybmFtZTp2ZXJ5IWxvbmchcGFzc3dvcmR0aGF0Y3JlYXRlc2Fsb25nYmFzaWM2NGVuY29kaW5n") in headers

def get_user_custom_headers_func():
  """Insert some custom http headers, including a duplicate."""
  headers = []
  headers.append(('key1', 'value1'))
  headers.append(('key1', 'value2'))
  headers.append(('key2', 'value3'))
  return headers


def get_unused_port():
  """ Find an unused port http://stackoverflow.com/questions/1365265 """
  with closing(socket.socket(socket.AF_INET, socket.SOCK_STREAM)) as s:
    s.bind(('', 0))
    s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
    return s.getsockname()[1]

def shutdown_server(server):
  """Helper method to shutdown a http server."""
  if server.httpd is not None:
    server.httpd.shutdown()
  if server.http_server_thread is not None:
    server.http_server_thread.join()

def count_tuples_with_key(tuple_list, key_to_count):
  """Counts the number of tuples in a list that have a specific key.
  Args:
    tuple_list: A list of key-value tuples.
    key_to_count: The key to count occurrences of.
  Returns:
    The number of tuples with the specified key.
  """
  count = 0
  for key, _ in tuple_list:
    if key == key_to_count:
      count += 1
  return count



#
# Autogenerated by Thrift Compiler (0.16.0)
#
# DO NOT EDIT UNLESS YOU ARE SURE THAT YOU KNOW WHAT YOU ARE DOING
#
#  options string: py:new_style,no_utf8strings
#

from thrift.Thrift import TType, TMessageType, TFrozenDict, TException, TApplicationException
from thrift.protocol.TProtocol import TProtocolException
from thrift.TRecursive import fix_spec

import impala._thrift_gen.ExecStats.ttypes
import impala._thrift_gen.Status.ttypes
import impala._thrift_gen.Types.ttypes
import impala._thrift_gen.beeswax.ttypes
import impala._thrift_gen.TCLIService.ttypes
import impala._thrift_gen.RuntimeProfile.ttypes

from thrift.transport import TTransport
all_structs = []


class TImpalaQueryOptions(object):
    ABORT_ON_ERROR = 0
    MAX_ERRORS = 1
    DISABLE_CODEGEN = 2
    BATCH_SIZE = 3
    MEM_LIMIT = 4
    NUM_NODES = 5
    MAX_SCAN_RANGE_LENGTH = 6
    MAX_IO_BUFFERS = 7
    NUM_SCANNER_THREADS = 8
    ALLOW_UNSUPPORTED_FORMATS = 9
    DEFAULT_ORDER_BY_LIMIT = 10
    DEBUG_ACTION = 11
    ABORT_ON_DEFAULT_LIMIT_EXCEEDED = 12
    COMPRESSION_CODEC = 13
    SEQ_COMPRESSION_MODE = 14
    HBASE_CACHING = 15
    HBASE_CACHE_BLOCKS = 16
    PARQUET_FILE_SIZE = 17
    EXPLAIN_LEVEL = 18
    SYNC_DDL = 19
    REQUEST_POOL = 20
    V_CPU_CORES = 21
    RESERVATION_REQUEST_TIMEOUT = 22
    DISABLE_CACHED_READS = 23
    DISABLE_OUTERMOST_TOPN = 24
    RM_INITIAL_MEM = 25
    QUERY_TIMEOUT_S = 26
    BUFFER_POOL_LIMIT = 27
    APPX_COUNT_DISTINCT = 28
    DISABLE_UNSAFE_SPILLS = 29
    EXEC_SINGLE_NODE_ROWS_THRESHOLD = 30
    OPTIMIZE_PARTITION_KEY_SCANS = 31
    REPLICA_PREFERENCE = 32
    SCHEDULE_RANDOM_REPLICA = 33
    SCAN_NODE_CODEGEN_THRESHOLD = 34
    DISABLE_STREAMING_PREAGGREGATIONS = 35
    RUNTIME_FILTER_MODE = 36
    RUNTIME_BLOOM_FILTER_SIZE = 37
    RUNTIME_FILTER_WAIT_TIME_MS = 38
    DISABLE_ROW_RUNTIME_FILTERING = 39
    MAX_NUM_RUNTIME_FILTERS = 40
    PARQUET_ANNOTATE_STRINGS_UTF8 = 41
    PARQUET_FALLBACK_SCHEMA_RESOLUTION = 42
    MT_DOP = 43
    S3_SKIP_INSERT_STAGING = 44
    RUNTIME_FILTER_MAX_SIZE = 45
    RUNTIME_FILTER_MIN_SIZE = 46
    PREFETCH_MODE = 47
    STRICT_MODE = 48
    SCRATCH_LIMIT = 49
    ENABLE_EXPR_REWRITES = 50
    DECIMAL_V2 = 51
    PARQUET_DICTIONARY_FILTERING = 52
    PARQUET_ARRAY_RESOLUTION = 53
    PARQUET_READ_STATISTICS = 54
    DEFAULT_JOIN_DISTRIBUTION_MODE = 55
    DISABLE_CODEGEN_ROWS_THRESHOLD = 56
    DEFAULT_SPILLABLE_BUFFER_SIZE = 57
    MIN_SPILLABLE_BUFFER_SIZE = 58
    MAX_ROW_SIZE = 59
    IDLE_SESSION_TIMEOUT = 60
    COMPUTE_STATS_MIN_SAMPLE_SIZE = 61
    EXEC_TIME_LIMIT_S = 62
    SHUFFLE_DISTINCT_EXPRS = 63
    MAX_MEM_ESTIMATE_FOR_ADMISSION = 64
    THREAD_RESERVATION_LIMIT = 65
    THREAD_RESERVATION_AGGREGATE_LIMIT = 66
    KUDU_READ_MODE = 67
    ALLOW_ERASURE_CODED_FILES = 68
    TIMEZONE = 69
    SCAN_BYTES_LIMIT = 70
    CPU_LIMIT_S = 71
    TOPN_BYTES_LIMIT = 72
    CLIENT_IDENTIFIER = 73
    RESOURCE_TRACE_RATIO = 74
    NUM_REMOTE_EXECUTOR_CANDIDATES = 75
    NUM_ROWS_PRODUCED_LIMIT = 76
    PLANNER_TESTCASE_MODE = 77
    DEFAULT_FILE_FORMAT = 78
    PARQUET_TIMESTAMP_TYPE = 79
    PARQUET_READ_PAGE_INDEX = 80
    PARQUET_WRITE_PAGE_INDEX = 81
    PARQUET_PAGE_ROW_COUNT_LIMIT = 82
    DISABLE_HDFS_NUM_ROWS_ESTIMATE = 83
    DEFAULT_HINTS_INSERT_STATEMENT = 84
    SPOOL_QUERY_RESULTS = 85
    DEFAULT_TRANSACTIONAL_TYPE = 86
    STATEMENT_EXPRESSION_LIMIT = 87
    MAX_STATEMENT_LENGTH_BYTES = 88
    DISABLE_DATA_CACHE = 89
    MAX_RESULT_SPOOLING_MEM = 90
    MAX_SPILLED_RESULT_SPOOLING_MEM = 91
    DISABLE_HBASE_NUM_ROWS_ESTIMATE = 92
    FETCH_ROWS_TIMEOUT_MS = 93
    NOW_STRING = 94
    PARQUET_OBJECT_STORE_SPLIT_SIZE = 95
    MEM_LIMIT_EXECUTORS = 96
    BROADCAST_BYTES_LIMIT = 97
    PREAGG_BYTES_LIMIT = 98
    ENABLE_CNF_REWRITES = 99
    MAX_CNF_EXPRS = 100
    KUDU_SNAPSHOT_READ_TIMESTAMP_MICROS = 101
    RETRY_FAILED_QUERIES = 102
    ENABLED_RUNTIME_FILTER_TYPES = 103
    ASYNC_CODEGEN = 104
    ENABLE_DISTINCT_SEMI_JOIN_OPTIMIZATION = 105
    SORT_RUN_BYTES_LIMIT = 106
    MAX_FS_WRITERS = 107
    REFRESH_UPDATED_HMS_PARTITIONS = 108
    SPOOL_ALL_RESULTS_FOR_RETRIES = 109
    RUNTIME_FILTER_ERROR_RATE = 110
    USE_LOCAL_TZ_FOR_UNIX_TIMESTAMP_CONVERSIONS = 111
    CONVERT_LEGACY_HIVE_PARQUET_UTC_TIMESTAMPS = 112
    ENABLE_OUTER_JOIN_TO_INNER_TRANSFORMATION = 113
    TARGETED_KUDU_SCAN_RANGE_LENGTH = 114
    REPORT_SKEW_LIMIT = 115
    OPTIMIZE_SIMPLE_LIMIT = 116
    USE_DOP_FOR_COSTING = 117
    BROADCAST_TO_PARTITION_FACTOR = 118
    JOIN_ROWS_PRODUCED_LIMIT = 119
    UTF8_MODE = 120
    ANALYTIC_RANK_PUSHDOWN_THRESHOLD = 121
    MINMAX_FILTER_THRESHOLD = 122
    MINMAX_FILTERING_LEVEL = 123
    COMPUTE_COLUMN_MINMAX_STATS = 124
    SHOW_COLUMN_MINMAX_STATS = 125
    DEFAULT_NDV_SCALE = 126
    KUDU_REPLICA_SELECTION = 127
    DELETE_STATS_IN_TRUNCATE = 128
    PARQUET_BLOOM_FILTERING = 129
    MINMAX_FILTER_SORTED_COLUMNS = 130
    MINMAX_FILTER_FAST_CODE_PATH = 131
    ENABLE_KUDU_TRANSACTION = 132
    MINMAX_FILTER_PARTITION_COLUMNS = 133
    PARQUET_BLOOM_FILTER_WRITE = 134
    ORC_READ_STATISTICS = 135
    ENABLE_ASYNC_DDL_EXECUTION = 136
    ENABLE_ASYNC_LOAD_DATA_EXECUTION = 137
    PARQUET_LATE_MATERIALIZATION_THRESHOLD = 138
    PARQUET_DICTIONARY_RUNTIME_FILTER_ENTRY_LIMIT = 139
    ABORT_JAVA_UDF_ON_EXCEPTION = 140
    ORC_ASYNC_READ = 141
    RUNTIME_IN_LIST_FILTER_ENTRY_LIMIT = 142
    ENABLE_REPLAN = 143
    TEST_REPLAN = 144
    LOCK_MAX_WAIT_TIME_S = 145
    ORC_SCHEMA_RESOLUTION = 146
    EXPAND_COMPLEX_TYPES = 147
    FALLBACK_DB_FOR_FUNCTIONS = 148
    DISABLE_CODEGEN_CACHE = 149
    CODEGEN_CACHE_MODE = 150
    STRINGIFY_MAP_KEYS = 151
    ENABLE_TRIVIAL_QUERY_FOR_ADMISSION = 152
    COMPUTE_PROCESSING_COST = 153
    PROCESSING_COST_MIN_THREADS = 154
    JOIN_SELECTIVITY_CORRELATION_FACTOR = 155
    MAX_FRAGMENT_INSTANCES_PER_NODE = 156
    MAX_SORT_RUN_SIZE = 157
    ALLOW_UNSAFE_CASTS = 158
    NUM_THREADS_FOR_TABLE_MIGRATION = 159
    DISABLE_OPTIMIZED_ICEBERG_V2_READ = 160
    VALUES_STMT_AVOID_LOSSY_CHAR_PADDING = 161
    LARGE_AGG_MEM_THRESHOLD = 162
    AGG_MEM_CORRELATION_FACTOR = 163
    MEM_LIMIT_COORDINATORS = 164
    ICEBERG_PREDICATE_PUSHDOWN_SUBSETTING = 165
    HDFS_SCANNER_NON_RESERVED_BYTES = 166
    CODEGEN_OPT_LEVEL = 167
    KUDU_TABLE_RESERVE_SECONDS = 168
    CONVERT_KUDU_UTC_TIMESTAMPS = 169
    DISABLE_KUDU_LOCAL_TIMESTAMP_BLOOM_FILTER = 170
    RUNTIME_FILTER_CARDINALITY_REDUCTION_SCALE = 171
    MAX_NUM_FILTERS_AGGREGATED_PER_HOST = 172
    QUERY_CPU_COUNT_DIVISOR = 173
    ENABLE_TUPLE_CACHE = 174
    ICEBERG_DISABLE_COUNT_STAR_OPTIMIZATION = 175
    RUNTIME_FILTER_IDS_TO_SKIP = 176
    SLOT_COUNT_STRATEGY = 177
    CLEAN_DBCP_DS_CACHE = 178
    USE_NULL_SLOTS_CACHE = 179
    WRITE_KUDU_UTC_TIMESTAMPS = 180
    DISABLE_OPTIMIZED_JSON_COUNT_STAR = 181
    LONG_POLLING_TIME_MS = 182
    ENABLE_TUPLE_CACHE_VERIFICATION = 183
    ENABLE_TUPLE_ANALYSIS_IN_AGGREGATE = 184
    ESTIMATE_DUPLICATE_IN_PREAGG = 185

    _VALUES_TO_NAMES = {
        0: "ABORT_ON_ERROR",
        1: "MAX_ERRORS",
        2: "DISABLE_CODEGEN",
        3: "BATCH_SIZE",
        4: "MEM_LIMIT",
        5: "NUM_NODES",
        6: "MAX_SCAN_RANGE_LENGTH",
        7: "MAX_IO_BUFFERS",
        8: "NUM_SCANNER_THREADS",
        9: "ALLOW_UNSUPPORTED_FORMATS",
        10: "DEFAULT_ORDER_BY_LIMIT",
        11: "DEBUG_ACTION",
        12: "ABORT_ON_DEFAULT_LIMIT_EXCEEDED",
        13: "COMPRESSION_CODEC",
        14: "SEQ_COMPRESSION_MODE",
        15: "HBASE_CACHING",
        16: "HBASE_CACHE_BLOCKS",
        17: "PARQUET_FILE_SIZE",
        18: "EXPLAIN_LEVEL",
        19: "SYNC_DDL",
        20: "REQUEST_POOL",
        21: "V_CPU_CORES",
        22: "RESERVATION_REQUEST_TIMEOUT",
        23: "DISABLE_CACHED_READS",
        24: "DISABLE_OUTERMOST_TOPN",
        25: "RM_INITIAL_MEM",
        26: "QUERY_TIMEOUT_S",
        27: "BUFFER_POOL_LIMIT",
        28: "APPX_COUNT_DISTINCT",
        29: "DISABLE_UNSAFE_SPILLS",
        30: "EXEC_SINGLE_NODE_ROWS_THRESHOLD",
        31: "OPTIMIZE_PARTITION_KEY_SCANS",
        32: "REPLICA_PREFERENCE",
        33: "SCHEDULE_RANDOM_REPLICA",
        34: "SCAN_NODE_CODEGEN_THRESHOLD",
        35: "DISABLE_STREAMING_PREAGGREGATIONS",
        36: "RUNTIME_FILTER_MODE",
        37: "RUNTIME_BLOOM_FILTER_SIZE",
        38: "RUNTIME_FILTER_WAIT_TIME_MS",
        39: "DISABLE_ROW_RUNTIME_FILTERING",
        40: "MAX_NUM_RUNTIME_FILTERS",
        41: "PARQUET_ANNOTATE_STRINGS_UTF8",
        42: "PARQUET_FALLBACK_SCHEMA_RESOLUTION",
        43: "MT_DOP",
        44: "S3_SKIP_INSERT_STAGING",
        45: "RUNTIME_FILTER_MAX_SIZE",
        46: "RUNTIME_FILTER_MIN_SIZE",
        47: "PREFETCH_MODE",
        48: "STRICT_MODE",
        49: "SCRATCH_LIMIT",
        50: "ENABLE_EXPR_REWRITES",
        51: "DECIMAL_V2",
        52: "PARQUET_DICTIONARY_FILTERING",
        53: "PARQUET_ARRAY_RESOLUTION",
        54: "PARQUET_READ_STATISTICS",
        55: "DEFAULT_JOIN_DISTRIBUTION_MODE",
        56: "DISABLE_CODEGEN_ROWS_THRESHOLD",
        57: "DEFAULT_SPILLABLE_BUFFER_SIZE",
        58: "MIN_SPILLABLE_BUFFER_SIZE",
        59: "MAX_ROW_SIZE",
        60: "IDLE_SESSION_TIMEOUT",
        61: "COMPUTE_STATS_MIN_SAMPLE_SIZE",
        62: "EXEC_TIME_LIMIT_S",
        63: "SHUFFLE_DISTINCT_EXPRS",
        64: "MAX_MEM_ESTIMATE_FOR_ADMISSION",
        65: "THREAD_RESERVATION_LIMIT",
        66: "THREAD_RESERVATION_AGGREGATE_LIMIT",
        67: "KUDU_READ_MODE",
        68: "ALLOW_ERASURE_CODED_FILES",
        69: "TIMEZONE",
        70: "SCAN_BYTES_LIMIT",
        71: "CPU_LIMIT_S",
        72: "TOPN_BYTES_LIMIT",
        73: "CLIENT_IDENTIFIER",
        74: "RESOURCE_TRACE_RATIO",
        75: "NUM_REMOTE_EXECUTOR_CANDIDATES",
        76: "NUM_ROWS_PRODUCED_LIMIT",
        77: "PLANNER_TESTCASE_MODE",
        78: "DEFAULT_FILE_FORMAT",
        79: "PARQUET_TIMESTAMP_TYPE",
        80: "PARQUET_READ_PAGE_INDEX",
        81: "PARQUET_WRITE_PAGE_INDEX",
        82: "PARQUET_PAGE_ROW_COUNT_LIMIT",
        83: "DISABLE_HDFS_NUM_ROWS_ESTIMATE",
        84: "DEFAULT_HINTS_INSERT_STATEMENT",
        85: "SPOOL_QUERY_RESULTS",
        86: "DEFAULT_TRANSACTIONAL_TYPE",
        87: "STATEMENT_EXPRESSION_LIMIT",
        88: "MAX_STATEMENT_LENGTH_BYTES",
        89: "DISABLE_DATA_CACHE",
        90: "MAX_RESULT_SPOOLING_MEM",
        91: "MAX_SPILLED_RESULT_SPOOLING_MEM",
        92: "DISABLE_HBASE_NUM_ROWS_ESTIMATE",
        93: "FETCH_ROWS_TIMEOUT_MS",
        94: "NOW_STRING",
        95: "PARQUET_OBJECT_STORE_SPLIT_SIZE",
        96: "MEM_LIMIT_EXECUTORS",
        97: "BROADCAST_BYTES_LIMIT",
        98: "PREAGG_BYTES_LIMIT",
        99: "ENABLE_CNF_REWRITES",
        100: "MAX_CNF_EXPRS",
        101: "KUDU_SNAPSHOT_READ_TIMESTAMP_MICROS",
        102: "RETRY_FAILED_QUERIES",
        103: "ENABLED_RUNTIME_FILTER_TYPES",
        104: "ASYNC_CODEGEN",
        105: "ENABLE_DISTINCT_SEMI_JOIN_OPTIMIZATION",
        106: "SORT_RUN_BYTES_LIMIT",
        107: "MAX_FS_WRITERS",
        108: "REFRESH_UPDATED_HMS_PARTITIONS",
        109: "SPOOL_ALL_RESULTS_FOR_RETRIES",
        110: "RUNTIME_FILTER_ERROR_RATE",
        111: "USE_LOCAL_TZ_FOR_UNIX_TIMESTAMP_CONVERSIONS",
        112: "CONVERT_LEGACY_HIVE_PARQUET_UTC_TIMESTAMPS",
        113: "ENABLE_OUTER_JOIN_TO_INNER_TRANSFORMATION",
        114: "TARGETED_KUDU_SCAN_RANGE_LENGTH",
        115: "REPORT_SKEW_LIMIT",
        116: "OPTIMIZE_SIMPLE_LIMIT",
        117: "USE_DOP_FOR_COSTING",
        118: "BROADCAST_TO_PARTITION_FACTOR",
        119: "JOIN_ROWS_PRODUCED_LIMIT",
        120: "UTF8_MODE",
        121: "ANALYTIC_RANK_PUSHDOWN_THRESHOLD",
        122: "MINMAX_FILTER_THRESHOLD",
        123: "MINMAX_FILTERING_LEVEL",
        124: "COMPUTE_COLUMN_MINMAX_STATS",
        125: "SHOW_COLUMN_MINMAX_STATS",
        126: "DEFAULT_NDV_SCALE",
        127: "KUDU_REPLICA_SELECTION",
        128: "DELETE_STATS_IN_TRUNCATE",
        129: "PARQUET_BLOOM_FILTERING",
        130: "MINMAX_FILTER_SORTED_COLUMNS",
        131: "MINMAX_FILTER_FAST_CODE_PATH",
        132: "ENABLE_KUDU_TRANSACTION",
        133: "MINMAX_FILTER_PARTITION_COLUMNS",
        134: "PARQUET_BLOOM_FILTER_WRITE",
        135: "ORC_READ_STATISTICS",
        136: "ENABLE_ASYNC_DDL_EXECUTION",
        137: "ENABLE_ASYNC_LOAD_DATA_EXECUTION",
        138: "PARQUET_LATE_MATERIALIZATION_THRESHOLD",
        139: "PARQUET_DICTIONARY_RUNTIME_FILTER_ENTRY_LIMIT",
        140: "ABORT_JAVA_UDF_ON_EXCEPTION",
        141: "ORC_ASYNC_READ",
        142: "RUNTIME_IN_LIST_FILTER_ENTRY_LIMIT",
        143: "ENABLE_REPLAN",
        144: "TEST_REPLAN",
        145: "LOCK_MAX_WAIT_TIME_S",
        146: "ORC_SCHEMA_RESOLUTION",
        147: "EXPAND_COMPLEX_TYPES",
        148: "FALLBACK_DB_FOR_FUNCTIONS",
        149: "DISABLE_CODEGEN_CACHE",
        150: "CODEGEN_CACHE_MODE",
        151: "STRINGIFY_MAP_KEYS",
        152: "ENABLE_TRIVIAL_QUERY_FOR_ADMISSION",
        153: "COMPUTE_PROCESSING_COST",
        154: "PROCESSING_COST_MIN_THREADS",
        155: "JOIN_SELECTIVITY_CORRELATION_FACTOR",
        156: "MAX_FRAGMENT_INSTANCES_PER_NODE",
        157: "MAX_SORT_RUN_SIZE",
        158: "ALLOW_UNSAFE_CASTS",
        159: "NUM_THREADS_FOR_TABLE_MIGRATION",
        160: "DISABLE_OPTIMIZED_ICEBERG_V2_READ",
        161: "VALUES_STMT_AVOID_LOSSY_CHAR_PADDING",
        162: "LARGE_AGG_MEM_THRESHOLD",
        163: "AGG_MEM_CORRELATION_FACTOR",
        164: "MEM_LIMIT_COORDINATORS",
        165: "ICEBERG_PREDICATE_PUSHDOWN_SUBSETTING",
        166: "HDFS_SCANNER_NON_RESERVED_BYTES",
        167: "CODEGEN_OPT_LEVEL",
        168: "KUDU_TABLE_RESERVE_SECONDS",
        169: "CONVERT_KUDU_UTC_TIMESTAMPS",
        170: "DISABLE_KUDU_LOCAL_TIMESTAMP_BLOOM_FILTER",
        171: "RUNTIME_FILTER_CARDINALITY_REDUCTION_SCALE",
        172: "MAX_NUM_FILTERS_AGGREGATED_PER_HOST",
        173: "QUERY_CPU_COUNT_DIVISOR",
        174: "ENABLE_TUPLE_CACHE",
        175: "ICEBERG_DISABLE_COUNT_STAR_OPTIMIZATION",
        176: "RUNTIME_FILTER_IDS_TO_SKIP",
        177: "SLOT_COUNT_STRATEGY",
        178: "CLEAN_DBCP_DS_CACHE",
        179: "USE_NULL_SLOTS_CACHE",
        180: "WRITE_KUDU_UTC_TIMESTAMPS",
        181: "DISABLE_OPTIMIZED_JSON_COUNT_STAR",
        182: "LONG_POLLING_TIME_MS",
        183: "ENABLE_TUPLE_CACHE_VERIFICATION",
        184: "ENABLE_TUPLE_ANALYSIS_IN_AGGREGATE",
        185: "ESTIMATE_DUPLICATE_IN_PREAGG",
    }

    _NAMES_TO_VALUES = {
        "ABORT_ON_ERROR": 0,
        "MAX_ERRORS": 1,
        "DISABLE_CODEGEN": 2,
        "BATCH_SIZE": 3,
        "MEM_LIMIT": 4,
        "NUM_NODES": 5,
        "MAX_SCAN_RANGE_LENGTH": 6,
        "MAX_IO_BUFFERS": 7,
        "NUM_SCANNER_THREADS": 8,
        "ALLOW_UNSUPPORTED_FORMATS": 9,
        "DEFAULT_ORDER_BY_LIMIT": 10,
        "DEBUG_ACTION": 11,
        "ABORT_ON_DEFAULT_LIMIT_EXCEEDED": 12,
        "COMPRESSION_CODEC": 13,
        "SEQ_COMPRESSION_MODE": 14,
        "HBASE_CACHING": 15,
        "HBASE_CACHE_BLOCKS": 16,
        "PARQUET_FILE_SIZE": 17,
        "EXPLAIN_LEVEL": 18,
        "SYNC_DDL": 19,
        "REQUEST_POOL": 20,
        "V_CPU_CORES": 21,
        "RESERVATION_REQUEST_TIMEOUT": 22,
        "DISABLE_CACHED_READS": 23,
        "DISABLE_OUTERMOST_TOPN": 24,
        "RM_INITIAL_MEM": 25,
        "QUERY_TIMEOUT_S": 26,
        "BUFFER_POOL_LIMIT": 27,
        "APPX_COUNT_DISTINCT": 28,
        "DISABLE_UNSAFE_SPILLS": 29,
        "EXEC_SINGLE_NODE_ROWS_THRESHOLD": 30,
        "OPTIMIZE_PARTITION_KEY_SCANS": 31,
        "REPLICA_PREFERENCE": 32,
        "SCHEDULE_RANDOM_REPLICA": 33,
        "SCAN_NODE_CODEGEN_THRESHOLD": 34,
        "DISABLE_STREAMING_PREAGGREGATIONS": 35,
        "RUNTIME_FILTER_MODE": 36,
        "RUNTIME_BLOOM_FILTER_SIZE": 37,
        "RUNTIME_FILTER_WAIT_TIME_MS": 38,
        "DISABLE_ROW_RUNTIME_FILTERING": 39,
        "MAX_NUM_RUNTIME_FILTERS": 40,
        "PARQUET_ANNOTATE_STRINGS_UTF8": 41,
        "PARQUET_FALLBACK_SCHEMA_RESOLUTION": 42,
        "MT_DOP": 43,
        "S3_SKIP_INSERT_STAGING": 44,
        "RUNTIME_FILTER_MAX_SIZE": 45,
        "RUNTIME_FILTER_MIN_SIZE": 46,
        "PREFETCH_MODE": 47,
        "STRICT_MODE": 48,
        "SCRATCH_LIMIT": 49,
        "ENABLE_EXPR_REWRITES": 50,
        "DECIMAL_V2": 51,
        "PARQUET_DICTIONARY_FILTERING": 52,
        "PARQUET_ARRAY_RESOLUTION": 53,
        "PARQUET_READ_STATISTICS": 54,
        "DEFAULT_JOIN_DISTRIBUTION_MODE": 55,
        "DISABLE_CODEGEN_ROWS_THRESHOLD": 56,
        "DEFAULT_SPILLABLE_BUFFER_SIZE": 57,
        "MIN_SPILLABLE_BUFFER_SIZE": 58,
        "MAX_ROW_SIZE": 59,
        "IDLE_SESSION_TIMEOUT": 60,
        "COMPUTE_STATS_MIN_SAMPLE_SIZE": 61,
        "EXEC_TIME_LIMIT_S": 62,
        "SHUFFLE_DISTINCT_EXPRS": 63,
        "MAX_MEM_ESTIMATE_FOR_ADMISSION": 64,
        "THREAD_RESERVATION_LIMIT": 65,
        "THREAD_RESERVATION_AGGREGATE_LIMIT": 66,
        "KUDU_READ_MODE": 67,
        "ALLOW_ERASURE_CODED_FILES": 68,
        "TIMEZONE": 69,
        "SCAN_BYTES_LIMIT": 70,
        "CPU_LIMIT_S": 71,
        "TOPN_BYTES_LIMIT": 72,
        "CLIENT_IDENTIFIER": 73,
        "RESOURCE_TRACE_RATIO": 74,
        "NUM_REMOTE_EXECUTOR_CANDIDATES": 75,
        "NUM_ROWS_PRODUCED_LIMIT": 76,
        "PLANNER_TESTCASE_MODE": 77,
        "DEFAULT_FILE_FORMAT": 78,
        "PARQUET_TIMESTAMP_TYPE": 79,
        "PARQUET_READ_PAGE_INDEX": 80,
        "PARQUET_WRITE_PAGE_INDEX": 81,
        "PARQUET_PAGE_ROW_COUNT_LIMIT": 82,
        "DISABLE_HDFS_NUM_ROWS_ESTIMATE": 83,
        "DEFAULT_HINTS_INSERT_STATEMENT": 84,
        "SPOOL_QUERY_RESULTS": 85,
        "DEFAULT_TRANSACTIONAL_TYPE": 86,
        "STATEMENT_EXPRESSION_LIMIT": 87,
        "MAX_STATEMENT_LENGTH_BYTES": 88,
        "DISABLE_DATA_CACHE": 89,
        "MAX_RESULT_SPOOLING_MEM": 90,
        "MAX_SPILLED_RESULT_SPOOLING_MEM": 91,
        "DISABLE_HBASE_NUM_ROWS_ESTIMATE": 92,
        "FETCH_ROWS_TIMEOUT_MS": 93,
        "NOW_STRING": 94,
        "PARQUET_OBJECT_STORE_SPLIT_SIZE": 95,
        "MEM_LIMIT_EXECUTORS": 96,
        "BROADCAST_BYTES_LIMIT": 97,
        "PREAGG_BYTES_LIMIT": 98,
        "ENABLE_CNF_REWRITES": 99,
        "MAX_CNF_EXPRS": 100,
        "KUDU_SNAPSHOT_READ_TIMESTAMP_MICROS": 101,
        "RETRY_FAILED_QUERIES": 102,
        "ENABLED_RUNTIME_FILTER_TYPES": 103,
        "ASYNC_CODEGEN": 104,
        "ENABLE_DISTINCT_SEMI_JOIN_OPTIMIZATION": 105,
        "SORT_RUN_BYTES_LIMIT": 106,
        "MAX_FS_WRITERS": 107,
        "REFRESH_UPDATED_HMS_PARTITIONS": 108,
        "SPOOL_ALL_RESULTS_FOR_RETRIES": 109,
        "RUNTIME_FILTER_ERROR_RATE": 110,
        "USE_LOCAL_TZ_FOR_UNIX_TIMESTAMP_CONVERSIONS": 111,
        "CONVERT_LEGACY_HIVE_PARQUET_UTC_TIMESTAMPS": 112,
        "ENABLE_OUTER_JOIN_TO_INNER_TRANSFORMATION": 113,
        "TARGETED_KUDU_SCAN_RANGE_LENGTH": 114,
        "REPORT_SKEW_LIMIT": 115,
        "OPTIMIZE_SIMPLE_LIMIT": 116,
        "USE_DOP_FOR_COSTING": 117,
        "BROADCAST_TO_PARTITION_FACTOR": 118,
        "JOIN_ROWS_PRODUCED_LIMIT": 119,
        "UTF8_MODE": 120,
        "ANALYTIC_RANK_PUSHDOWN_THRESHOLD": 121,
        "MINMAX_FILTER_THRESHOLD": 122,
        "MINMAX_FILTERING_LEVEL": 123,
        "COMPUTE_COLUMN_MINMAX_STATS": 124,
        "SHOW_COLUMN_MINMAX_STATS": 125,
        "DEFAULT_NDV_SCALE": 126,
        "KUDU_REPLICA_SELECTION": 127,
        "DELETE_STATS_IN_TRUNCATE": 128,
        "PARQUET_BLOOM_FILTERING": 129,
        "MINMAX_FILTER_SORTED_COLUMNS": 130,
        "MINMAX_FILTER_FAST_CODE_PATH": 131,
        "ENABLE_KUDU_TRANSACTION": 132,
        "MINMAX_FILTER_PARTITION_COLUMNS": 133,
        "PARQUET_BLOOM_FILTER_WRITE": 134,
        "ORC_READ_STATISTICS": 135,
        "ENABLE_ASYNC_DDL_EXECUTION": 136,
        "ENABLE_ASYNC_LOAD_DATA_EXECUTION": 137,
        "PARQUET_LATE_MATERIALIZATION_THRESHOLD": 138,
        "PARQUET_DICTIONARY_RUNTIME_FILTER_ENTRY_LIMIT": 139,
        "ABORT_JAVA_UDF_ON_EXCEPTION": 140,
        "ORC_ASYNC_READ": 141,
        "RUNTIME_IN_LIST_FILTER_ENTRY_LIMIT": 142,
        "ENABLE_REPLAN": 143,
        "TEST_REPLAN": 144,
        "LOCK_MAX_WAIT_TIME_S": 145,
        "ORC_SCHEMA_RESOLUTION": 146,
        "EXPAND_COMPLEX_TYPES": 147,
        "FALLBACK_DB_FOR_FUNCTIONS": 148,
        "DISABLE_CODEGEN_CACHE": 149,
        "CODEGEN_CACHE_MODE": 150,
        "STRINGIFY_MAP_KEYS": 151,
        "ENABLE_TRIVIAL_QUERY_FOR_ADMISSION": 152,
        "COMPUTE_PROCESSING_COST": 153,
        "PROCESSING_COST_MIN_THREADS": 154,
        "JOIN_SELECTIVITY_CORRELATION_FACTOR": 155,
        "MAX_FRAGMENT_INSTANCES_PER_NODE": 156,
        "MAX_SORT_RUN_SIZE": 157,
        "ALLOW_UNSAFE_CASTS": 158,
        "NUM_THREADS_FOR_TABLE_MIGRATION": 159,
        "DISABLE_OPTIMIZED_ICEBERG_V2_READ": 160,
        "VALUES_STMT_AVOID_LOSSY_CHAR_PADDING": 161,
        "LARGE_AGG_MEM_THRESHOLD": 162,
        "AGG_MEM_CORRELATION_FACTOR": 163,
        "MEM_LIMIT_COORDINATORS": 164,
        "ICEBERG_PREDICATE_PUSHDOWN_SUBSETTING": 165,
        "HDFS_SCANNER_NON_RESERVED_BYTES": 166,
        "CODEGEN_OPT_LEVEL": 167,
        "KUDU_TABLE_RESERVE_SECONDS": 168,
        "CONVERT_KUDU_UTC_TIMESTAMPS": 169,
        "DISABLE_KUDU_LOCAL_TIMESTAMP_BLOOM_FILTER": 170,
        "RUNTIME_FILTER_CARDINALITY_REDUCTION_SCALE": 171,
        "MAX_NUM_FILTERS_AGGREGATED_PER_HOST": 172,
        "QUERY_CPU_COUNT_DIVISOR": 173,
        "ENABLE_TUPLE_CACHE": 174,
        "ICEBERG_DISABLE_COUNT_STAR_OPTIMIZATION": 175,
        "RUNTIME_FILTER_IDS_TO_SKIP": 176,
        "SLOT_COUNT_STRATEGY": 177,
        "CLEAN_DBCP_DS_CACHE": 178,
        "USE_NULL_SLOTS_CACHE": 179,
        "WRITE_KUDU_UTC_TIMESTAMPS": 180,
        "DISABLE_OPTIMIZED_JSON_COUNT_STAR": 181,
        "LONG_POLLING_TIME_MS": 182,
        "ENABLE_TUPLE_CACHE_VERIFICATION": 183,
        "ENABLE_TUPLE_ANALYSIS_IN_AGGREGATE": 184,
        "ESTIMATE_DUPLICATE_IN_PREAGG": 185,
    }


class TDmlResult(object):
    """
    Attributes:
     - rows_modified
     - rows_deleted
     - num_row_errors

    """


    def __init__(self, rows_modified=None, rows_deleted=None, num_row_errors=None,):
        self.rows_modified = rows_modified
        self.rows_deleted = rows_deleted
        self.num_row_errors = num_row_errors

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.MAP:
                    self.rows_modified = {}
                    (_ktype1, _vtype2, _size0) = iprot.readMapBegin()
                    for _i4 in range(_size0):
                        _key5 = iprot.readString()
                        _val6 = iprot.readI64()
                        self.rows_modified[_key5] = _val6
                    iprot.readMapEnd()
                else:
                    iprot.skip(ftype)
            elif fid == 3:
                if ftype == TType.MAP:
                    self.rows_deleted = {}
                    (_ktype8, _vtype9, _size7) = iprot.readMapBegin()
                    for _i11 in range(_size7):
                        _key12 = iprot.readString()
                        _val13 = iprot.readI64()
                        self.rows_deleted[_key12] = _val13
                    iprot.readMapEnd()
                else:
                    iprot.skip(ftype)
            elif fid == 2:
                if ftype == TType.I64:
                    self.num_row_errors = iprot.readI64()
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('TDmlResult')
        if self.rows_modified is not None:
            oprot.writeFieldBegin('rows_modified', TType.MAP, 1)
            oprot.writeMapBegin(TType.STRING, TType.I64, len(self.rows_modified))
            for kiter14, viter15 in self.rows_modified.items():
                oprot.writeString(kiter14)
                oprot.writeI64(viter15)
            oprot.writeMapEnd()
            oprot.writeFieldEnd()
        if self.num_row_errors is not None:
            oprot.writeFieldBegin('num_row_errors', TType.I64, 2)
            oprot.writeI64(self.num_row_errors)
            oprot.writeFieldEnd()
        if self.rows_deleted is not None:
            oprot.writeFieldBegin('rows_deleted', TType.MAP, 3)
            oprot.writeMapBegin(TType.STRING, TType.I64, len(self.rows_deleted))
            for kiter16, viter17 in self.rows_deleted.items():
                oprot.writeString(kiter16)
                oprot.writeI64(viter17)
            oprot.writeMapEnd()
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        if self.rows_modified is None:
            raise TProtocolException(message='Required field rows_modified is unset!')
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)


class TPingImpalaServiceResp(object):
    """
    Attributes:
     - version
     - webserver_address

    """


    def __init__(self, version=None, webserver_address=None,):
        self.version = version
        self.webserver_address = webserver_address

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.STRING:
                    self.version = iprot.readString()
                else:
                    iprot.skip(ftype)
            elif fid == 2:
                if ftype == TType.STRING:
                    self.webserver_address = iprot.readString()
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('TPingImpalaServiceResp')
        if self.version is not None:
            oprot.writeFieldBegin('version', TType.STRING, 1)
            oprot.writeString(self.version)
            oprot.writeFieldEnd()
        if self.webserver_address is not None:
            oprot.writeFieldBegin('webserver_address', TType.STRING, 2)
            oprot.writeString(self.webserver_address)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)


class TResetTableReq(object):
    """
    Attributes:
     - db_name
     - table_name

    """


    def __init__(self, db_name=None, table_name=None,):
        self.db_name = db_name
        self.table_name = table_name

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.STRING:
                    self.db_name = iprot.readString()
                else:
                    iprot.skip(ftype)
            elif fid == 2:
                if ftype == TType.STRING:
                    self.table_name = iprot.readString()
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('TResetTableReq')
        if self.db_name is not None:
            oprot.writeFieldBegin('db_name', TType.STRING, 1)
            oprot.writeString(self.db_name)
            oprot.writeFieldEnd()
        if self.table_name is not None:
            oprot.writeFieldBegin('table_name', TType.STRING, 2)
            oprot.writeString(self.table_name)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        if self.db_name is None:
            raise TProtocolException(message='Required field db_name is unset!')
        if self.table_name is None:
            raise TProtocolException(message='Required field table_name is unset!')
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)


class TPingImpalaHS2ServiceReq(object):
    """
    Attributes:
     - sessionHandle

    """


    def __init__(self, sessionHandle=None,):
        self.sessionHandle = sessionHandle

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.STRUCT:
                    self.sessionHandle = impala._thrift_gen.TCLIService.ttypes.TSessionHandle()
                    self.sessionHandle.read(iprot)
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('TPingImpalaHS2ServiceReq')
        if self.sessionHandle is not None:
            oprot.writeFieldBegin('sessionHandle', TType.STRUCT, 1)
            self.sessionHandle.write(oprot)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        if self.sessionHandle is None:
            raise TProtocolException(message='Required field sessionHandle is unset!')
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)


class TPingImpalaHS2ServiceResp(object):
    """
    Attributes:
     - status
     - version
     - webserver_address
     - timestamp

    """


    def __init__(self, status=None, version=None, webserver_address=None, timestamp=None,):
        self.status = status
        self.version = version
        self.webserver_address = webserver_address
        self.timestamp = timestamp

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.STRUCT:
                    self.status = impala._thrift_gen.TCLIService.ttypes.TStatus()
                    self.status.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 2:
                if ftype == TType.STRING:
                    self.version = iprot.readString()
                else:
                    iprot.skip(ftype)
            elif fid == 3:
                if ftype == TType.STRING:
                    self.webserver_address = iprot.readString()
                else:
                    iprot.skip(ftype)
            elif fid == 4:
                if ftype == TType.I64:
                    self.timestamp = iprot.readI64()
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('TPingImpalaHS2ServiceResp')
        if self.status is not None:
            oprot.writeFieldBegin('status', TType.STRUCT, 1)
            self.status.write(oprot)
            oprot.writeFieldEnd()
        if self.version is not None:
            oprot.writeFieldBegin('version', TType.STRING, 2)
            oprot.writeString(self.version)
            oprot.writeFieldEnd()
        if self.webserver_address is not None:
            oprot.writeFieldBegin('webserver_address', TType.STRING, 3)
            oprot.writeString(self.webserver_address)
            oprot.writeFieldEnd()
        if self.timestamp is not None:
            oprot.writeFieldBegin('timestamp', TType.I64, 4)
            oprot.writeI64(self.timestamp)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        if self.status is None:
            raise TProtocolException(message='Required field status is unset!')
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)


class TCloseImpalaOperationReq(object):
    """
    Attributes:
     - operationHandle

    """


    def __init__(self, operationHandle=None,):
        self.operationHandle = operationHandle

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.STRUCT:
                    self.operationHandle = impala._thrift_gen.TCLIService.ttypes.TOperationHandle()
                    self.operationHandle.read(iprot)
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('TCloseImpalaOperationReq')
        if self.operationHandle is not None:
            oprot.writeFieldBegin('operationHandle', TType.STRUCT, 1)
            self.operationHandle.write(oprot)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        if self.operationHandle is None:
            raise TProtocolException(message='Required field operationHandle is unset!')
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)


class TCloseImpalaOperationResp(object):
    """
    Attributes:
     - status
     - dml_result

    """


    def __init__(self, status=None, dml_result=None,):
        self.status = status
        self.dml_result = dml_result

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.STRUCT:
                    self.status = impala._thrift_gen.TCLIService.ttypes.TStatus()
                    self.status.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 2:
                if ftype == TType.STRUCT:
                    self.dml_result = TDmlResult()
                    self.dml_result.read(iprot)
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('TCloseImpalaOperationResp')
        if self.status is not None:
            oprot.writeFieldBegin('status', TType.STRUCT, 1)
            self.status.write(oprot)
            oprot.writeFieldEnd()
        if self.dml_result is not None:
            oprot.writeFieldBegin('dml_result', TType.STRUCT, 2)
            self.dml_result.write(oprot)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        if self.status is None:
            raise TProtocolException(message='Required field status is unset!')
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)


class TGetExecSummaryReq(object):
    """
    Attributes:
     - operationHandle
     - sessionHandle
     - include_query_attempts

    """


    def __init__(self, operationHandle=None, sessionHandle=None, include_query_attempts=False,):
        self.operationHandle = operationHandle
        self.sessionHandle = sessionHandle
        self.include_query_attempts = include_query_attempts

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.STRUCT:
                    self.operationHandle = impala._thrift_gen.TCLIService.ttypes.TOperationHandle()
                    self.operationHandle.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 2:
                if ftype == TType.STRUCT:
                    self.sessionHandle = impala._thrift_gen.TCLIService.ttypes.TSessionHandle()
                    self.sessionHandle.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 3:
                if ftype == TType.BOOL:
                    self.include_query_attempts = iprot.readBool()
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('TGetExecSummaryReq')
        if self.operationHandle is not None:
            oprot.writeFieldBegin('operationHandle', TType.STRUCT, 1)
            self.operationHandle.write(oprot)
            oprot.writeFieldEnd()
        if self.sessionHandle is not None:
            oprot.writeFieldBegin('sessionHandle', TType.STRUCT, 2)
            self.sessionHandle.write(oprot)
            oprot.writeFieldEnd()
        if self.include_query_attempts is not None:
            oprot.writeFieldBegin('include_query_attempts', TType.BOOL, 3)
            oprot.writeBool(self.include_query_attempts)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)


class TGetExecSummaryResp(object):
    """
    Attributes:
     - status
     - summary
     - failed_summaries

    """


    def __init__(self, status=None, summary=None, failed_summaries=None,):
        self.status = status
        self.summary = summary
        self.failed_summaries = failed_summaries

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.STRUCT:
                    self.status = impala._thrift_gen.TCLIService.ttypes.TStatus()
                    self.status.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 2:
                if ftype == TType.STRUCT:
                    self.summary = impala._thrift_gen.ExecStats.ttypes.TExecSummary()
                    self.summary.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 3:
                if ftype == TType.LIST:
                    self.failed_summaries = []
                    (_etype21, _size18) = iprot.readListBegin()
                    for _i22 in range(_size18):
                        _elem23 = impala._thrift_gen.ExecStats.ttypes.TExecSummary()
                        _elem23.read(iprot)
                        self.failed_summaries.append(_elem23)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('TGetExecSummaryResp')
        if self.status is not None:
            oprot.writeFieldBegin('status', TType.STRUCT, 1)
            self.status.write(oprot)
            oprot.writeFieldEnd()
        if self.summary is not None:
            oprot.writeFieldBegin('summary', TType.STRUCT, 2)
            self.summary.write(oprot)
            oprot.writeFieldEnd()
        if self.failed_summaries is not None:
            oprot.writeFieldBegin('failed_summaries', TType.LIST, 3)
            oprot.writeListBegin(TType.STRUCT, len(self.failed_summaries))
            for iter24 in self.failed_summaries:
                iter24.write(oprot)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        if self.status is None:
            raise TProtocolException(message='Required field status is unset!')
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)


class TGetRuntimeProfileReq(object):
    """
    Attributes:
     - operationHandle
     - sessionHandle
     - format
     - include_query_attempts

    """


    def __init__(self, operationHandle=None, sessionHandle=None, format=0, include_query_attempts=False,):
        self.operationHandle = operationHandle
        self.sessionHandle = sessionHandle
        self.format = format
        self.include_query_attempts = include_query_attempts

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.STRUCT:
                    self.operationHandle = impala._thrift_gen.TCLIService.ttypes.TOperationHandle()
                    self.operationHandle.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 2:
                if ftype == TType.STRUCT:
                    self.sessionHandle = impala._thrift_gen.TCLIService.ttypes.TSessionHandle()
                    self.sessionHandle.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 3:
                if ftype == TType.I32:
                    self.format = iprot.readI32()
                else:
                    iprot.skip(ftype)
            elif fid == 4:
                if ftype == TType.BOOL:
                    self.include_query_attempts = iprot.readBool()
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('TGetRuntimeProfileReq')
        if self.operationHandle is not None:
            oprot.writeFieldBegin('operationHandle', TType.STRUCT, 1)
            self.operationHandle.write(oprot)
            oprot.writeFieldEnd()
        if self.sessionHandle is not None:
            oprot.writeFieldBegin('sessionHandle', TType.STRUCT, 2)
            self.sessionHandle.write(oprot)
            oprot.writeFieldEnd()
        if self.format is not None:
            oprot.writeFieldBegin('format', TType.I32, 3)
            oprot.writeI32(self.format)
            oprot.writeFieldEnd()
        if self.include_query_attempts is not None:
            oprot.writeFieldBegin('include_query_attempts', TType.BOOL, 4)
            oprot.writeBool(self.include_query_attempts)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)


class TGetRuntimeProfileResp(object):
    """
    Attributes:
     - status
     - profile
     - thrift_profile
     - failed_profiles
     - failed_thrift_profiles

    """


    def __init__(self, status=None, profile=None, thrift_profile=None, failed_profiles=None, failed_thrift_profiles=None,):
        self.status = status
        self.profile = profile
        self.thrift_profile = thrift_profile
        self.failed_profiles = failed_profiles
        self.failed_thrift_profiles = failed_thrift_profiles

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.STRUCT:
                    self.status = impala._thrift_gen.TCLIService.ttypes.TStatus()
                    self.status.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 2:
                if ftype == TType.STRING:
                    self.profile = iprot.readString()
                else:
                    iprot.skip(ftype)
            elif fid == 3:
                if ftype == TType.STRUCT:
                    self.thrift_profile = impala._thrift_gen.RuntimeProfile.ttypes.TRuntimeProfileTree()
                    self.thrift_profile.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 4:
                if ftype == TType.LIST:
                    self.failed_profiles = []
                    (_etype28, _size25) = iprot.readListBegin()
                    for _i29 in range(_size25):
                        _elem30 = iprot.readString()
                        self.failed_profiles.append(_elem30)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            elif fid == 5:
                if ftype == TType.LIST:
                    self.failed_thrift_profiles = []
                    (_etype34, _size31) = iprot.readListBegin()
                    for _i35 in range(_size31):
                        _elem36 = impala._thrift_gen.RuntimeProfile.ttypes.TRuntimeProfileTree()
                        _elem36.read(iprot)
                        self.failed_thrift_profiles.append(_elem36)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('TGetRuntimeProfileResp')
        if self.status is not None:
            oprot.writeFieldBegin('status', TType.STRUCT, 1)
            self.status.write(oprot)
            oprot.writeFieldEnd()
        if self.profile is not None:
            oprot.writeFieldBegin('profile', TType.STRING, 2)
            oprot.writeString(self.profile)
            oprot.writeFieldEnd()
        if self.thrift_profile is not None:
            oprot.writeFieldBegin('thrift_profile', TType.STRUCT, 3)
            self.thrift_profile.write(oprot)
            oprot.writeFieldEnd()
        if self.failed_profiles is not None:
            oprot.writeFieldBegin('failed_profiles', TType.LIST, 4)
            oprot.writeListBegin(TType.STRING, len(self.failed_profiles))
            for iter37 in self.failed_profiles:
                oprot.writeString(iter37)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        if self.failed_thrift_profiles is not None:
            oprot.writeFieldBegin('failed_thrift_profiles', TType.LIST, 5)
            oprot.writeListBegin(TType.STRUCT, len(self.failed_thrift_profiles))
            for iter38 in self.failed_thrift_profiles:
                iter38.write(oprot)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        if self.status is None:
            raise TProtocolException(message='Required field status is unset!')
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(TDmlResult)
TDmlResult.thrift_spec = (
    None,  # 0
    (1, TType.MAP, 'rows_modified', (TType.STRING, None, TType.I64, None, False), None, ),  # 1
    (2, TType.I64, 'num_row_errors', None, None, ),  # 2
    (3, TType.MAP, 'rows_deleted', (TType.STRING, None, TType.I64, None, False), None, ),  # 3
)
all_structs.append(TPingImpalaServiceResp)
TPingImpalaServiceResp.thrift_spec = (
    None,  # 0
    (1, TType.STRING, 'version', None, None, ),  # 1
    (2, TType.STRING, 'webserver_address', None, None, ),  # 2
)
all_structs.append(TResetTableReq)
TResetTableReq.thrift_spec = (
    None,  # 0
    (1, TType.STRING, 'db_name', None, None, ),  # 1
    (2, TType.STRING, 'table_name', None, None, ),  # 2
)
all_structs.append(TPingImpalaHS2ServiceReq)
TPingImpalaHS2ServiceReq.thrift_spec = (
    None,  # 0
    (1, TType.STRUCT, 'sessionHandle', [impala._thrift_gen.TCLIService.ttypes.TSessionHandle, None], None, ),  # 1
)
all_structs.append(TPingImpalaHS2ServiceResp)
TPingImpalaHS2ServiceResp.thrift_spec = (
    None,  # 0
    (1, TType.STRUCT, 'status', [impala._thrift_gen.TCLIService.ttypes.TStatus, None], None, ),  # 1
    (2, TType.STRING, 'version', None, None, ),  # 2
    (3, TType.STRING, 'webserver_address', None, None, ),  # 3
    (4, TType.I64, 'timestamp', None, None, ),  # 4
)
all_structs.append(TCloseImpalaOperationReq)
TCloseImpalaOperationReq.thrift_spec = (
    None,  # 0
    (1, TType.STRUCT, 'operationHandle', [impala._thrift_gen.TCLIService.ttypes.TOperationHandle, None], None, ),  # 1
)
all_structs.append(TCloseImpalaOperationResp)
TCloseImpalaOperationResp.thrift_spec = (
    None,  # 0
    (1, TType.STRUCT, 'status', [impala._thrift_gen.TCLIService.ttypes.TStatus, None], None, ),  # 1
    (2, TType.STRUCT, 'dml_result', [TDmlResult, None], None, ),  # 2
)
all_structs.append(TGetExecSummaryReq)
TGetExecSummaryReq.thrift_spec = (
    None,  # 0
    (1, TType.STRUCT, 'operationHandle', [impala._thrift_gen.TCLIService.ttypes.TOperationHandle, None], None, ),  # 1
    (2, TType.STRUCT, 'sessionHandle', [impala._thrift_gen.TCLIService.ttypes.TSessionHandle, None], None, ),  # 2
    (3, TType.BOOL, 'include_query_attempts', None, False, ),  # 3
)
all_structs.append(TGetExecSummaryResp)
TGetExecSummaryResp.thrift_spec = (
    None,  # 0
    (1, TType.STRUCT, 'status', [impala._thrift_gen.TCLIService.ttypes.TStatus, None], None, ),  # 1
    (2, TType.STRUCT, 'summary', [impala._thrift_gen.ExecStats.ttypes.TExecSummary, None], None, ),  # 2
    (3, TType.LIST, 'failed_summaries', (TType.STRUCT, [impala._thrift_gen.ExecStats.ttypes.TExecSummary, None], False), None, ),  # 3
)
all_structs.append(TGetRuntimeProfileReq)
TGetRuntimeProfileReq.thrift_spec = (
    None,  # 0
    (1, TType.STRUCT, 'operationHandle', [impala._thrift_gen.TCLIService.ttypes.TOperationHandle, None], None, ),  # 1
    (2, TType.STRUCT, 'sessionHandle', [impala._thrift_gen.TCLIService.ttypes.TSessionHandle, None], None, ),  # 2
    (3, TType.I32, 'format', None, 0, ),  # 3
    (4, TType.BOOL, 'include_query_attempts', None, False, ),  # 4
)
all_structs.append(TGetRuntimeProfileResp)
TGetRuntimeProfileResp.thrift_spec = (
    None,  # 0
    (1, TType.STRUCT, 'status', [impala._thrift_gen.TCLIService.ttypes.TStatus, None], None, ),  # 1
    (2, TType.STRING, 'profile', None, None, ),  # 2
    (3, TType.STRUCT, 'thrift_profile', [impala._thrift_gen.RuntimeProfile.ttypes.TRuntimeProfileTree, None], None, ),  # 3
    (4, TType.LIST, 'failed_profiles', (TType.STRING, None, False), None, ),  # 4
    (5, TType.LIST, 'failed_thrift_profiles', (TType.STRUCT, [impala._thrift_gen.RuntimeProfile.ttypes.TRuntimeProfileTree, None], False), None, ),  # 5
)
fix_spec(all_structs)
del all_structs



#
# Autogenerated by Thrift Compiler (0.16.0)
#
# DO NOT EDIT UNLESS YOU ARE SURE THAT YOU KNOW WHAT YOU ARE DOING
#
#  options string: py:new_style,no_utf8strings
#

from thrift.Thrift import TType, TMessageType, TFrozenDict, TException, TApplicationException
from thrift.protocol.TProtocol import TProtocolException
from thrift.TRecursive import fix_spec

import impala._thrift_gen.hive_metastore.ttypes

from thrift.transport import TTransport
all_structs = []


class QueryState(object):
    CREATED = 0
    INITIALIZED = 1
    COMPILED = 2
    RUNNING = 3
    FINISHED = 4
    EXCEPTION = 5

    _VALUES_TO_NAMES = {
        0: "CREATED",
        1: "INITIALIZED",
        2: "COMPILED",
        3: "RUNNING",
        4: "FINISHED",
        5: "EXCEPTION",
    }

    _NAMES_TO_VALUES = {
        "CREATED": 0,
        "INITIALIZED": 1,
        "COMPILED": 2,
        "RUNNING": 3,
        "FINISHED": 4,
        "EXCEPTION": 5,
    }


class TQueryOptionLevel(object):
    REGULAR = 0
    ADVANCED = 1
    DEVELOPMENT = 2
    DEPRECATED = 3
    REMOVED = 4

    _VALUES_TO_NAMES = {
        0: "REGULAR",
        1: "ADVANCED",
        2: "DEVELOPMENT",
        3: "DEPRECATED",
        4: "REMOVED",
    }

    _NAMES_TO_VALUES = {
        "REGULAR": 0,
        "ADVANCED": 1,
        "DEVELOPMENT": 2,
        "DEPRECATED": 3,
        "REMOVED": 4,
    }


class Query(object):
    """
    Attributes:
     - query
     - configuration
     - hadoop_user

    """


    def __init__(self, query=None, configuration=None, hadoop_user=None,):
        self.query = query
        self.configuration = configuration
        self.hadoop_user = hadoop_user

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.STRING:
                    self.query = iprot.readString()
                else:
                    iprot.skip(ftype)
            elif fid == 3:
                if ftype == TType.LIST:
                    self.configuration = []
                    (_etype3, _size0) = iprot.readListBegin()
                    for _i4 in range(_size0):
                        _elem5 = iprot.readString()
                        self.configuration.append(_elem5)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            elif fid == 4:
                if ftype == TType.STRING:
                    self.hadoop_user = iprot.readString()
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('Query')
        if self.query is not None:
            oprot.writeFieldBegin('query', TType.STRING, 1)
            oprot.writeString(self.query)
            oprot.writeFieldEnd()
        if self.configuration is not None:
            oprot.writeFieldBegin('configuration', TType.LIST, 3)
            oprot.writeListBegin(TType.STRING, len(self.configuration))
            for iter6 in self.configuration:
                oprot.writeString(iter6)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        if self.hadoop_user is not None:
            oprot.writeFieldBegin('hadoop_user', TType.STRING, 4)
            oprot.writeString(self.hadoop_user)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)


class QueryHandle(object):
    """
    Attributes:
     - id
     - log_context

    """


    def __init__(self, id=None, log_context=None,):
        self.id = id
        self.log_context = log_context

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.STRING:
                    self.id = iprot.readString()
                else:
                    iprot.skip(ftype)
            elif fid == 2:
                if ftype == TType.STRING:
                    self.log_context = iprot.readString()
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('QueryHandle')
        if self.id is not None:
            oprot.writeFieldBegin('id', TType.STRING, 1)
            oprot.writeString(self.id)
            oprot.writeFieldEnd()
        if self.log_context is not None:
            oprot.writeFieldBegin('log_context', TType.STRING, 2)
            oprot.writeString(self.log_context)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)


class QueryExplanation(object):
    """
    Attributes:
     - textual

    """


    def __init__(self, textual=None,):
        self.textual = textual

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.STRING:
                    self.textual = iprot.readString()
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('QueryExplanation')
        if self.textual is not None:
            oprot.writeFieldBegin('textual', TType.STRING, 1)
            oprot.writeString(self.textual)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)


class Results(object):
    """
    Attributes:
     - ready
     - columns
     - data
     - start_row
     - has_more

    """


    def __init__(self, ready=None, columns=None, data=None, start_row=None, has_more=None,):
        self.ready = ready
        self.columns = columns
        self.data = data
        self.start_row = start_row
        self.has_more = has_more

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.BOOL:
                    self.ready = iprot.readBool()
                else:
                    iprot.skip(ftype)
            elif fid == 2:
                if ftype == TType.LIST:
                    self.columns = []
                    (_etype10, _size7) = iprot.readListBegin()
                    for _i11 in range(_size7):
                        _elem12 = iprot.readString()
                        self.columns.append(_elem12)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            elif fid == 3:
                if ftype == TType.LIST:
                    self.data = []
                    (_etype16, _size13) = iprot.readListBegin()
                    for _i17 in range(_size13):
                        _elem18 = iprot.readString()
                        self.data.append(_elem18)
                    iprot.readListEnd()
                else:
                    iprot.skip(ftype)
            elif fid == 4:
                if ftype == TType.I64:
                    self.start_row = iprot.readI64()
                else:
                    iprot.skip(ftype)
            elif fid == 5:
                if ftype == TType.BOOL:
                    self.has_more = iprot.readBool()
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('Results')
        if self.ready is not None:
            oprot.writeFieldBegin('ready', TType.BOOL, 1)
            oprot.writeBool(self.ready)
            oprot.writeFieldEnd()
        if self.columns is not None:
            oprot.writeFieldBegin('columns', TType.LIST, 2)
            oprot.writeListBegin(TType.STRING, len(self.columns))
            for iter19 in self.columns:
                oprot.writeString(iter19)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        if self.data is not None:
            oprot.writeFieldBegin('data', TType.LIST, 3)
            oprot.writeListBegin(TType.STRING, len(self.data))
            for iter20 in self.data:
                oprot.writeString(iter20)
            oprot.writeListEnd()
            oprot.writeFieldEnd()
        if self.start_row is not None:
            oprot.writeFieldBegin('start_row', TType.I64, 4)
            oprot.writeI64(self.start_row)
            oprot.writeFieldEnd()
        if self.has_more is not None:
            oprot.writeFieldBegin('has_more', TType.BOOL, 5)
            oprot.writeBool(self.has_more)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)


class ResultsMetadata(object):
    """
    Metadata information about the results.
    Applicable only for SELECT.

    Attributes:
     - schema: The schema of the results
     - table_dir: The directory containing the results. Not applicable for partition table.
     - in_tablename: If the results are straight from an existing table, the table name.
     - delim: Field delimiter

    """


    def __init__(self, schema=None, table_dir=None, in_tablename=None, delim=None,):
        self.schema = schema
        self.table_dir = table_dir
        self.in_tablename = in_tablename
        self.delim = delim

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.STRUCT:
                    self.schema = impala._thrift_gen.hive_metastore.ttypes.Schema()
                    self.schema.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 2:
                if ftype == TType.STRING:
                    self.table_dir = iprot.readString()
                else:
                    iprot.skip(ftype)
            elif fid == 3:
                if ftype == TType.STRING:
                    self.in_tablename = iprot.readString()
                else:
                    iprot.skip(ftype)
            elif fid == 4:
                if ftype == TType.STRING:
                    self.delim = iprot.readString()
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('ResultsMetadata')
        if self.schema is not None:
            oprot.writeFieldBegin('schema', TType.STRUCT, 1)
            self.schema.write(oprot)
            oprot.writeFieldEnd()
        if self.table_dir is not None:
            oprot.writeFieldBegin('table_dir', TType.STRING, 2)
            oprot.writeString(self.table_dir)
            oprot.writeFieldEnd()
        if self.in_tablename is not None:
            oprot.writeFieldBegin('in_tablename', TType.STRING, 3)
            oprot.writeString(self.in_tablename)
            oprot.writeFieldEnd()
        if self.delim is not None:
            oprot.writeFieldBegin('delim', TType.STRING, 4)
            oprot.writeString(self.delim)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)


class BeeswaxException(TException):
    """
    Attributes:
     - message
     - log_context
     - handle
     - errorCode
     - SQLState

    """


    def __init__(self, message=None, log_context=None, handle=None, errorCode=0, SQLState="     ",):
        super(BeeswaxException, self).__setattr__('message', message)
        super(BeeswaxException, self).__setattr__('log_context', log_context)
        super(BeeswaxException, self).__setattr__('handle', handle)
        super(BeeswaxException, self).__setattr__('errorCode', errorCode)
        super(BeeswaxException, self).__setattr__('SQLState', SQLState)

    def __setattr__(self, *args):
        raise TypeError("can't modify immutable instance")

    def __delattr__(self, *args):
        raise TypeError("can't modify immutable instance")

    def __hash__(self):
        return hash(self.__class__) ^ hash((self.message, self.log_context, self.handle, self.errorCode, self.SQLState, ))

    @classmethod
    def read(cls, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and cls.thrift_spec is not None:
            return iprot._fast_decode(None, iprot, [cls, cls.thrift_spec])
        iprot.readStructBegin()
        message = None
        log_context = None
        handle = None
        errorCode = 0
        SQLState = "     "
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.STRING:
                    message = iprot.readString()
                else:
                    iprot.skip(ftype)
            elif fid == 2:
                if ftype == TType.STRING:
                    log_context = iprot.readString()
                else:
                    iprot.skip(ftype)
            elif fid == 3:
                if ftype == TType.STRUCT:
                    handle = QueryHandle()
                    handle.read(iprot)
                else:
                    iprot.skip(ftype)
            elif fid == 4:
                if ftype == TType.I32:
                    errorCode = iprot.readI32()
                else:
                    iprot.skip(ftype)
            elif fid == 5:
                if ftype == TType.STRING:
                    SQLState = iprot.readString()
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()
        return cls(
            message=message,
            log_context=log_context,
            handle=handle,
            errorCode=errorCode,
            SQLState=SQLState,
        )

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('BeeswaxException')
        if self.message is not None:
            oprot.writeFieldBegin('message', TType.STRING, 1)
            oprot.writeString(self.message)
            oprot.writeFieldEnd()
        if self.log_context is not None:
            oprot.writeFieldBegin('log_context', TType.STRING, 2)
            oprot.writeString(self.log_context)
            oprot.writeFieldEnd()
        if self.handle is not None:
            oprot.writeFieldBegin('handle', TType.STRUCT, 3)
            self.handle.write(oprot)
            oprot.writeFieldEnd()
        if self.errorCode is not None:
            oprot.writeFieldBegin('errorCode', TType.I32, 4)
            oprot.writeI32(self.errorCode)
            oprot.writeFieldEnd()
        if self.SQLState is not None:
            oprot.writeFieldBegin('SQLState', TType.STRING, 5)
            oprot.writeString(self.SQLState)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __str__(self):
        return repr(self)

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)


class QueryNotFoundException(TException):


    def __setattr__(self, *args):
        raise TypeError("can't modify immutable instance")

    def __delattr__(self, *args):
        raise TypeError("can't modify immutable instance")

    def __hash__(self):
        return hash(self.__class__) ^ hash(())

    @classmethod
    def read(cls, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and cls.thrift_spec is not None:
            return iprot._fast_decode(None, iprot, [cls, cls.thrift_spec])
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()
        return cls(
        )

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('QueryNotFoundException')
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __str__(self):
        return repr(self)

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)


class ConfigVariable(object):
    """
    Represents a Hadoop-style configuration variable.

    Attributes:
     - key
     - value
     - description
     - level

    """


    def __init__(self, key=None, value=None, description=None, level=None,):
        self.key = key
        self.value = value
        self.description = description
        self.level = level

    def read(self, iprot):
        if iprot._fast_decode is not None and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None:
            iprot._fast_decode(self, iprot, [self.__class__, self.thrift_spec])
            return
        iprot.readStructBegin()
        while True:
            (fname, ftype, fid) = iprot.readFieldBegin()
            if ftype == TType.STOP:
                break
            if fid == 1:
                if ftype == TType.STRING:
                    self.key = iprot.readString()
                else:
                    iprot.skip(ftype)
            elif fid == 2:
                if ftype == TType.STRING:
                    self.value = iprot.readString()
                else:
                    iprot.skip(ftype)
            elif fid == 3:
                if ftype == TType.STRING:
                    self.description = iprot.readString()
                else:
                    iprot.skip(ftype)
            elif fid == 4:
                if ftype == TType.I32:
                    self.level = iprot.readI32()
                else:
                    iprot.skip(ftype)
            else:
                iprot.skip(ftype)
            iprot.readFieldEnd()
        iprot.readStructEnd()

    def write(self, oprot):
        if oprot._fast_encode is not None and self.thrift_spec is not None:
            oprot.trans.write(oprot._fast_encode(self, [self.__class__, self.thrift_spec]))
            return
        oprot.writeStructBegin('ConfigVariable')
        if self.key is not None:
            oprot.writeFieldBegin('key', TType.STRING, 1)
            oprot.writeString(self.key)
            oprot.writeFieldEnd()
        if self.value is not None:
            oprot.writeFieldBegin('value', TType.STRING, 2)
            oprot.writeString(self.value)
            oprot.writeFieldEnd()
        if self.description is not None:
            oprot.writeFieldBegin('description', TType.STRING, 3)
            oprot.writeString(self.description)
            oprot.writeFieldEnd()
        if self.level is not None:
            oprot.writeFieldBegin('level', TType.I32, 4)
            oprot.writeI32(self.level)
            oprot.writeFieldEnd()
        oprot.writeFieldStop()
        oprot.writeStructEnd()

    def validate(self):
        return

    def __repr__(self):
        L = ['%s=%r' % (key, value)
             for key, value in self.__dict__.items()]
        return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

    def __eq__(self, other):
        return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

    def __ne__(self, other):
        return not (self == other)
all_structs.append(Query)
Query.thrift_spec = (
    None,  # 0
    (1, TType.STRING, 'query', None, None, ),  # 1
    None,  # 2
    (3, TType.LIST, 'configuration', (TType.STRING, None, False), None, ),  # 3
    (4, TType.STRING, 'hadoop_user', None, None, ),  # 4
)
all_structs.append(QueryHandle)
QueryHandle.thrift_spec = (
    None,  # 0
    (1, TType.STRING, 'id', None, None, ),  # 1
    (2, TType.STRING, 'log_context', None, None, ),  # 2
)
all_structs.append(QueryExplanation)
QueryExplanation.thrift_spec = (
    None,  # 0
    (1, TType.STRING, 'textual', None, None, ),  # 1
)
all_structs.append(Results)
Results.thrift_spec = (
    None,  # 0
    (1, TType.BOOL, 'ready', None, None, ),  # 1
    (2, TType.LIST, 'columns', (TType.STRING, None, False), None, ),  # 2
    (3, TType.LIST, 'data', (TType.STRING, None, False), None, ),  # 3
    (4, TType.I64, 'start_row', None, None, ),  # 4
    (5, TType.BOOL, 'has_more', None, None, ),  # 5
)
all_structs.append(ResultsMetadata)
ResultsMetadata.thrift_spec = (
    None,  # 0
    (1, TType.STRUCT, 'schema', [impala._thrift_gen.hive_metastore.ttypes.Schema, None], None, ),  # 1
    (2, TType.STRING, 'table_dir', None, None, ),  # 2
    (3, TType.STRING, 'in_tablename', None, None, ),  # 3
    (4, TType.STRING, 'delim', None, None, ),  # 4
)
all_structs.append(BeeswaxException)
BeeswaxException.thrift_spec = (
    None,  # 0
    (1, TType.STRING, 'message', None, None, ),  # 1
    (2, TType.STRING, 'log_context', None, None, ),  # 2
    (3, TType.STRUCT, 'handle', [QueryHandle, None], None, ),  # 3
    (4, TType.I32, 'errorCode', None, 0, ),  # 4
    (5, TType.STRING, 'SQLState', None, "     ", ),  # 5
)
all_structs.append(QueryNotFoundException)
QueryNotFoundException.thrift_spec = (
)
all_structs.append(ConfigVariable)
ConfigVariable.thrift_spec = (
    None,  # 0
    (1, TType.STRING, 'key', None, None, ),  # 1
    (2, TType.STRING, 'value', None, None, ),  # 2
    (3, TType.STRING, 'description', None, None, ),  # 3
    (4, TType.I32, 'level', None, None, ),  # 4
)
fix_spec(all_structs)
del all_structs



# Copyright 2014 Cloudera Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# some inspiration from Dropbox's PyHive

from __future__ import absolute_import

import re

from sqlalchemy.dialects import registry

from sqlalchemy.engine.default import DefaultDialect, DefaultExecutionContext
from sqlalchemy.sql.compiler import (DDLCompiler, GenericTypeCompiler,
                                     IdentifierPreparer)
from sqlalchemy.types import (BOOLEAN, SMALLINT, BIGINT, TIMESTAMP, FLOAT,
                              DECIMAL, Integer, Float, String, CHAR, VARCHAR,
                              DATE)
from sqlalchemy import text


registry.register('impala', 'impala.sqlalchemy', 'ImpalaDialect')
registry.register('impala4', 'impala.sqlalchemy', 'Impala4Dialect')


class TINYINT(Integer):
    __visit_name__ = 'TINYINT'


class INT(Integer):
    __visit_name__ = 'INT'


class DOUBLE(Float):
    __visit_name__ = 'DOUBLE'


class STRING(String):
    __visit_name__ = 'STRING'


class ImpalaDDLCompiler(DDLCompiler):
    # Omit support for foreign keys due to Impala's limited support - https://issues.apache.org/jira/browse/IMPALA-2112
    def visit_foreign_key_constraint(self, constraint):
        return None

    # Omit support for primary keys due to Impala's limited support - https://issues.apache.org/jira/browse/IMPALA-2112
    def visit_primary_key_constraint(self, constraint):
        return None

    def get_column_specification(self, column, **kwargs):
        column.nullable = True  # Prevent adding NOT NULL constraints; since all columns in Impala are nullable
        colspec = super(ImpalaDDLCompiler, self).get_column_specification(
            column, **kwargs
        )
        return colspec

    def post_create_table(self, table):
        """Build table-level CREATE options."""

        table_opts = []

        if 'impala_partitioned_by' in table.kwargs:
            table_opts.append('PARTITIONED BY %s' % table.kwargs.get('impala_partitioned_by'))

        if 'impala_partition_by' in table.kwargs:
            table_opts.append('PARTITION BY %s' % table.kwargs.get('impala_partition_by'))

        if 'impala_stored_as' in table.kwargs:
            table_opts.append('STORED AS %s' % table.kwargs.get('impala_stored_as'))

        if 'impala_table_properties' in table.kwargs:
            table_properties = ["'{0}' = '{1}'".format(property_, value)
                                for property_, value
                                in sorted(table.kwargs.get('impala_table_properties', {}).items())]
            table_opts.append('TBLPROPERTIES (%s)' % ', '.join(table_properties))
        return '\n%s' % '\n'.join(table_opts)


# Impala type compiler for Impala <= 3.4.0
class ImpalaTypeCompiler(GenericTypeCompiler):
    # pylint: disable=unused-argument

    # https://docs.cloudera.com/documentation/enterprise/6/6.3/topics/impala_porting.html
    def visit_TEXT(self, type_):
        return 'STRING'

    visit_NCHAR = visit_TEXT
    visit_NVARCHAR = visit_TEXT

    def visit_DATETIME(self, type_):
        return 'TIMESTAMP'

    # Impala <= 3.4.0 only supports the TIMESTAMP type
    visit_TIME = visit_DATETIME
    visit_DATE = visit_DATETIME

    def visit_TINYINT(self, type_):
        return 'TINYINT'

    def visit_INT(self, type_):
        return 'INT'

    def visit_DOUBLE(self, type_):
        return 'DOUBLE'

    def visit_STRING(self, type_):
        return 'STRING'


# Impala type compiler for Impala >= 4.0
class Impala4TypeCompiler(ImpalaTypeCompiler):
    # Impala >= 4.0 supports the DATE type - https://issues.apache.org/jira/browse/IMPALA-6169
    def visit_DATE(self, type_):
        return 'DATE'

class ImpalaIdentifierPreparer(IdentifierPreparer):
    # https://github.com/cloudera/Impala/blob/master/fe/src/main/jflex/sql-scanner.flex
    reserved_words = frozenset([
        'add', 'aggregate', 'all', 'alter', 'analytic', 'and', 'anti',
        'api_version', 'array', 'as', 'asc', 'avro', 'between', 'bigint',
        'binary', 'boolean', 'by', 'cached', 'case', 'cast', 'change', 'char',
        'class', 'close_fn', 'column', 'columns', 'comment', 'compute',
        'create', 'cross', 'current', 'data', 'database', 'databases', 'date',
        'datetime', 'decimal', 'delimited', 'desc', 'describe', 'distinct',
        'div', 'double', 'drop', 'else', 'end', 'escaped', 'exists', 'explain',
        'external', 'false', 'fields', 'fileformat', 'finalize_fn', 'first',
        'float', 'following', 'for', 'format', 'formatted', 'from', 'full',
        'function', 'functions', 'grant', 'group', 'having', 'if', 'in',
        'init_fn', 'inner', 'inpath', 'insert', 'int', 'integer',
        'intermediate', 'interval', 'into', 'invalidate', 'is', 'join', 'last',
        'left', 'like', 'limit', 'lines', 'load', 'location', 'map',
        'merge_fn', 'metadata', 'not', 'null', 'nulls', 'offset', 'on', 'or',
        'order', 'outer', 'over', 'overwrite', 'parquet', 'parquetfile',
        'partition', 'partitioned', 'partitions', 'preceding', 'prepare_fn',
        'produced', 'range', 'rcfile', 'real', 'refresh', 'regexp', 'rename',
        'replace', 'returns', 'revoke', 'right', 'rlike', 'role', 'roles',
        'row', 'rows', 'schema', 'schemas', 'select', 'semi', 'sequencefile',
        'serdeproperties', 'serialize_fn', 'set', 'show', 'smallint', 'stats',
        'stored', 'straight_join', 'string', 'struct', 'symbol', 'table',
        'tables', 'tblproperties', 'terminated', 'textfile', 'then',
        'timestamp', 'tinyint', 'to', 'true', 'unbounded', 'uncached', 'union',
        'update_fn', 'use', 'using', 'values', 'varchar', 'view', 'when',
        'where', 'with'])

    legal_characters = re.compile(r'^[A-Z0-9_]+$', re.I)

    def __init__(self, dialect):
        super(ImpalaIdentifierPreparer, self).__init__(dialect,
                                                       initial_quote='`')


_impala_type_to_sqlalchemy_type = {
    'BOOLEAN': BOOLEAN,
    'TINYINT': TINYINT,
    'SMALLINT': SMALLINT,
    'INT': INT,
    'BIGINT': BIGINT,
    'TIMESTAMP': TIMESTAMP,
    'FLOAT': FLOAT,
    'DOUBLE': DOUBLE,
    'STRING': STRING,
    'DECIMAL': DECIMAL,
    'CHAR': CHAR,
    'VARCHAR': VARCHAR,
    'DATE': DATE
    }

class ImpalaExecutionContext(DefaultExecutionContext):
       def create_cursor(self):
           self._is_server_side = False
           cursor_configuration = self.execution_options.get('cursor_configuration', {})
           return self._dbapi_connection.cursor(configuration=cursor_configuration)


# Dialect to be used with Impala <= 3.4.0
class ImpalaDialect(DefaultDialect):
    name = 'impala'
    driver = 'impala'
    paramstyle = 'pyformat'
    preparer = ImpalaIdentifierPreparer
    max_identifier_length = 128
    supports_sane_rowcount = False
    supports_sane_multi_rowcount = False
    supports_multivalues_insert = True
    supports_sequences = False
    supports_native_decimal = True
    supports_native_boolean = True
    supports_native_enum = False
    supports_default_values = False
    returns_unicode_strings = True
    ddl_compiler = ImpalaDDLCompiler
    type_compiler = ImpalaTypeCompiler
    execution_ctx_cls = ImpalaExecutionContext

    @classmethod
    def dbapi(cls):
        # pylint: disable=method-hidden
        import impala.dbapi
        return impala.dbapi

    def create_connect_args(self, url):
        kwargs = {
            'host': url.host,
            'port': url.port,
            'user': url.username,
            'password': url.password,
            'database': url.database,
        }
        kwargs.update(url.query)
        return ([], kwargs)

    def initialize(self, connection):
        self.default_schema_name = connection.connection.default_db

    def _get_server_version_info(self, connection):
        raw = connection.execute(text('select version()')).scalar()
        v = raw.split()[2]
        m = re.match(r'.*?(\d{1,3})\.(\d{1,3})\.(\d{1,3}).*', v)
        return tuple([int(x) for x in m.group(1, 2, 3) if x is not None])

    def has_table(self, connection, table_name, schema=None, **kw):
        tables = self.get_table_names(connection, schema)
        if table_name in tables:
            return True
        return False

    def get_table_names(self, connection, schema=None, **kw):
        query = 'SHOW TABLES'
        if schema is not None:
            escaped_schema = re.sub(r"^[\`]{1,}|[\`]{1,}$", '', schema)
            escaped_schema = re.sub(r"^|$", '`', escaped_schema)
            query += ' IN %s' % escaped_schema
        tables = [
            tup[1] if len(tup) > 1 else tup[0]
            for tup in connection.execute(text(query)).fetchall()
        ]
        return tables

    def get_view_names(self, connection, schema=None, **kw):
        # Impala doesn't distinguish between tables and view when calling
        # SHOW TABLES. So return a blank list.
        return []

    def get_schema_names(self, connection, **kw):
        rp = connection.execute(text("SHOW SCHEMAS"))
        return [r[0] for r in rp]

    def get_columns(self, connection, table_name, schema=None, **kwargs):
        # pylint: disable=unused-argument
        name = table_name
        if schema is not None:
            name = '%s.%s' % (schema, name)
        query = 'SELECT * FROM %s LIMIT 0' % name
        cursor = connection.execute(text(query))
        schema = cursor.cursor.description
        # We need to fetch the empty results otherwise these queries remain in
        # flight
        cursor.fetchall()
        column_info = []
        # `select * from table` results in 'tablename.columnname', so strip off 'tablename.'
        for col in schema:
            column_info.append({
                'name': col[0].split('.')[-1].strip() if '.' in col[0] else col[0],
                'type': _impala_type_to_sqlalchemy_type[col[1]],
                'nullable': True,
                'autoincrement': False})
        return column_info

    def get_pk_constraint(self, connection, table_name, schema=None, **kw):
        # pylint: disable=unused-argument
        # no primary keys in impala
        return {'constrained_columns': [], 'name': None}

    def get_foreign_keys(self, connection, table_name, schema=None, **kw):
        # no foreign keys in impala
        return []

    def get_indexes(self, connection, table_name, schema=None, **kw):
        # no indexes in impala
        # TODO(laserson): handle partitions, like in PyHive
        return []

    def do_rollback(self, dbapi_connection):
        # no transactions in impala
        pass


# Dialect to be used with Impala >= 4.0
class Impala4Dialect(ImpalaDialect):
    name = 'impala4'
    driver = 'impala4'
    type_compiler = Impala4TypeCompiler
